#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008, 2011 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent
from pykickstart.version import versionToLongString, RHEL5, RHEL6, FC3, FC4, FC5, F29
from pykickstart.version import F7, F9, F12, F13, F14, F15, F18, F23, F25, RHEL8
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class FC3_RaidData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.device = kwargs.get("device", None)
        self.fstype = kwargs.get("fstype", "")
        self.level = kwargs.get("level", "")
        self.format = kwargs.get("format", True)
        self.spares = kwargs.get("spares", 0)
        self.preexist = kwargs.get("preexist", False)
        self.mountpoint = kwargs.get("mountpoint", "")
        self.members = kwargs.get("members", [])

    def __eq__(self, y):
        if not y:
            return False

        return self.device == y.device

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""

        # NB: using str(device) b/c when device=0 (as int) the condition fails
        # I'm not sure if we want to modify the tests b/c they compare device=md0 to device=0
        # and expect both to be equal
        if str(self.device):
            retval += " --device=%s" % self.device
        if self.fstype:
            retval += " --fstype=\"%s\"" % self.fstype
        if self.level:
            retval += " --level=%s" % self.level.upper()
        if not self.format:
            retval += " --noformat"
        if self.spares != 0:
            retval += " --spares=%d" % self.spares
        if self.preexist:
            retval += " --useexisting"

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "raid %s%s" % (self.mountpoint, self._getArgsAsStr())

        # Do not output the members list if --preexist was passed in.
        # This would be invalid input according to the parse method.
        if not self.preexist:
            retval += " " + " ".join(self.members)

        return retval.strip() + "\n"

class FC4_RaidData(FC3_RaidData):
    removedKeywords = FC3_RaidData.removedKeywords
    removedAttrs = FC3_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC3_RaidData.__init__(self, *args, **kwargs)
        self.fsopts = kwargs.get("fsopts", "")

    def _getArgsAsStr(self):
        retval = FC3_RaidData._getArgsAsStr(self)

        if self.fsopts:
            retval += " --fsoptions=\"%s\"" % self.fsopts

        return retval

class FC5_RaidData(FC4_RaidData):
    removedKeywords = FC4_RaidData.removedKeywords
    removedAttrs = FC4_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC4_RaidData.__init__(self, *args, **kwargs)
        self.bytesPerInode = kwargs.get("bytesPerInode", 4096)

    def _getArgsAsStr(self):
        retval = FC4_RaidData._getArgsAsStr(self)

        if hasattr(self, "bytesPerInode") and self.bytesPerInode != 0:
            retval += " --bytes-per-inode=%d" % self.bytesPerInode

        return retval

class RHEL5_RaidData(FC5_RaidData):
    removedKeywords = FC5_RaidData.removedKeywords
    removedAttrs = FC5_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC5_RaidData.__init__(self, *args, **kwargs)
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

    def _getArgsAsStr(self):
        retval = FC5_RaidData._getArgsAsStr(self)

        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase:
                retval += " --passphrase=\"%s\"" % self.passphrase

        return retval

class F7_RaidData(FC5_RaidData):
    pass

class F9_RaidData(F7_RaidData):
    removedKeywords = F7_RaidData.removedKeywords + ["bytesPerInode"]
    removedAttrs = F7_RaidData.removedAttrs + ["bytesPerInode"]

    def __init__(self, *args, **kwargs):
        F7_RaidData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.fsprofile = kwargs.get("fsprofile", "")
        self.encrypted = kwargs.get("encrypted", False)
        self.passphrase = kwargs.get("passphrase", "")

    def _getArgsAsStr(self):
        retval = F7_RaidData._getArgsAsStr(self)

        if self.fsprofile:
            retval += " --fsprofile=\"%s\"" % self.fsprofile
        if self.encrypted:
            retval += " --encrypted"

            if self.passphrase:
                retval += " --passphrase=\"%s\"" % self.passphrase

        return retval

class F12_RaidData(F9_RaidData):
    removedKeywords = F9_RaidData.removedKeywords
    removedAttrs = F9_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F9_RaidData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.escrowcert = kwargs.get("escrowcert", "")
        self.backuppassphrase = kwargs.get("backuppassphrase", False)

    def _getArgsAsStr(self):
        retval = F9_RaidData._getArgsAsStr(self)

        if self.encrypted and self.escrowcert:
            retval += " --escrowcert=\"%s\"" % self.escrowcert

            if self.backuppassphrase:
                retval += " --backuppassphrase"
        return retval

class F13_RaidData(F12_RaidData):
    pass

class RHEL6_RaidData(F13_RaidData):
    removedKeywords = F13_RaidData.removedKeywords
    removedAttrs = F13_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_RaidData.__init__(self, *args, **kwargs)

        self.cipher = kwargs.get("cipher", "")

    def _getArgsAsStr(self):
        retval = F13_RaidData._getArgsAsStr(self)

        if self.encrypted and self.cipher:
            retval += " --cipher=\"%s\"" % self.cipher

        return retval

class F14_RaidData(F13_RaidData):
    pass

class F15_RaidData(F14_RaidData):
    removedKeywords = F14_RaidData.removedKeywords
    removedAttrs = F14_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F14_RaidData.__init__(self, *args, **kwargs)
        self.deleteRemovedAttrs()

        self.label = kwargs.get("label", "")

    def _getArgsAsStr(self):
        retval = F14_RaidData._getArgsAsStr(self)

        if self.label:
            retval += " --label=%s" % self.label

        return retval

class F18_RaidData(F15_RaidData):
    removedKeywords = F15_RaidData.removedKeywords
    removedAttrs = F15_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F15_RaidData.__init__(self, *args, **kwargs)

        self.cipher = kwargs.get("cipher", "")

    def _getArgsAsStr(self):
        retval = F15_RaidData._getArgsAsStr(self)

        if self.encrypted and self.cipher:
            retval += " --cipher=\"%s\"" % self.cipher

        return retval

class F23_RaidData(F18_RaidData):
    removedKeywords = F18_RaidData.removedKeywords
    removedAttrs = F18_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F18_RaidData.__init__(self, *args, **kwargs)
        self.mkfsopts = kwargs.get("mkfsoptions", "") or kwargs.get("mkfsopts", "")

    def _getArgsAsStr(self):
        retval = F18_RaidData._getArgsAsStr(self)

        if self.mkfsopts:
            retval += " --mkfsoptions=\"%s\"" % self.mkfsopts

        return retval

class F25_RaidData(F23_RaidData):
    removedKeywords = F23_RaidData.removedKeywords
    removedAttrs = F23_RaidData.removedAttrs

    def __init__(self, *args, **kwargs):
        F23_RaidData.__init__(self, *args, **kwargs)
        self.chunk_size = kwargs.get("chunk_size", None)

    def _getArgsAsStr(self):
        retval = F23_RaidData._getArgsAsStr(self)

        if self.chunk_size:
            retval += " --chunksize=%d" % self.chunk_size

        return retval

class RHEL7_RaidData(F25_RaidData):
    pass

class F29_RaidData(F25_RaidData):
    def __init__(self, *args, **kwargs):
        F25_RaidData.__init__(self, *args, **kwargs)
        self.luks_version = kwargs.get("luks_version", "")
        self.pbkdf = kwargs.get("pbkdf", "")
        self.pbkdf_memory = kwargs.get("pbkdf_memory", 0)
        self.pbkdf_time = kwargs.get("pbkdf_time", 0)
        self.pbkdf_iterations = kwargs.get("pbkdf_iterations", 0)

    def _getArgsAsStr(self):
        retval = F25_RaidData._getArgsAsStr(self)

        if self.encrypted and self.luks_version:
            retval += " --luks-version=%s" % self.luks_version

        if self.encrypted and self.pbkdf:
            retval += " --pbkdf=%s" % self.pbkdf

        if self.encrypted and self.pbkdf_memory:
            retval += " --pbkdf-memory=%s" % self.pbkdf_memory

        if self.encrypted and self.pbkdf_time:
            retval += " --pbkdf-time=%s" % self.pbkdf_time

        if self.encrypted and self.pbkdf_iterations:
            retval += " --pbkdf-iterations=%s" % self.pbkdf_iterations

        return retval

class RHEL8_RaidData(F29_RaidData):
    pass

class FC3_Raid(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=131, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        # A dict of all the RAID levels we support.  This means that if we
        # support more levels in the future, subclasses don't have to
        # duplicate too much.
        self.levelMap = {"RAID0": "RAID0", "0": "RAID0",
                         "RAID1": "RAID1", "1": "RAID1",
                         "RAID5": "RAID5", "5": "RAID5",
                         "RAID6": "RAID6", "6": "RAID6"}

        self.raidList = kwargs.get("raidList", [])
        self.op = self._getParser()

    def __str__(self):
        retval = ""

        for raid in self.raidList:
            retval += raid.__str__()

        return retval

    def _getParser(self):
        def device_cb(value):
            if value[0:2] == "md":
                return value[2:]
            else:
                return value

        def level_cb(value):
            if value.upper() in self.levelMap:
                return self.levelMap[value.upper()]
            else:
                raise KickstartParseError(_("Invalid raid level: %s") % value, lineno=self.lineno)

        op = KSOptionParser(prog="raid", description="""
                            Assembles a software RAID device.""",
                            epilog="""
                            The following example shows how to create a RAID
                            level 1 partition for /, and a RAID level 5 for
                            /usr, assuming there are three disks on the
                            system. It also creates three swap partitions, one
                            on each drive::

                                part raid.01 --size=6000 --ondisk=sda
                                part raid.02 --size=6000 --ondisk=sdb
                                part raid.03 --size=6000 --ondisk=sdc

                                part swap1 --size=512 --ondisk=sda
                                part swap2 --size=512 --ondisk=sdb
                                part swap3 --size=512 --ondisk=sdc

                                part raid.11 --size=6000 --ondisk=sda
                                part raid.12 --size=6000 --ondisk=sdb
                                part raid.13 --size=6000 --ondisk=sdc

                                raid / --level=1 --device=md0 raid.01 raid.02 raid.03
                                raid /usr --level=5 --device=md1 raid.11 raid.12 raid.13
                            """, version=FC3)
        op.add_argument("mntpoint", metavar="<mntpoint>", nargs=1, version=FC3,
                        help="""
                        Location where the RAID file system is mounted. If it
                        is /, the RAID level must be 1 unless a boot partition
                        (/boot) is present. If a boot partition is present, the
                        /boot partition must be level 1 and the root (/)
                        partition can be any of the available types.""")
        op.add_argument("partitions", metavar="<partitions*>", nargs="*",
                        version=FC3, help="""
                        The software raid partitions lists the RAID identifiers
                        to add to the RAID array.""")
        op.add_argument("--device", type=device_cb, required=True,
                        version=FC3, help="""
                        Name of the RAID device to use (such as 'fedora-root'
                        or 'home'). As of Fedora 19, RAID devices are no longer
                        referred to by names like 'md0'. If you have an old
                        (v0.90 metadata) array that you cannot assign a name to,
                        you can specify the array by a filesystem label or UUID
                        (eg: --device=LABEL=fedora-root).""")
        op.add_argument("--fstype", version=FC3, help="""
                        Sets the file system type for the RAID array. Valid
                        values include ext4, ext3, ext2, btrfs, swap, and vfat.
                        Other filesystems may be valid depending on command
                        line arguments passed to anaconda to enable other
                        filesystems.""")
        op.add_argument("--level", type=level_cb, version=FC3, help="""
                        RAID level to use %s.""" % set(self.levelMap.values()))
        op.add_argument("--noformat", dest="format", action="store_false",
                        default=True, version=FC3, help="""
                        Use an existing RAID device and do not format the RAID
                        array.""")
        op.add_argument("--spares", type=int, default=0, version=FC3, help="""
                        Specifies the number of spare drives allocated for the
                        RAID array. Spare drives are used to rebuild the array
                        in case of drive failure.""")
        op.add_argument("--useexisting", dest="preexist", action="store_true",
                        default=False, version=FC3, help="""
                        Use an existing RAID device and reformat it.""")
        return op

    def _getDevice(self, s):
        """ Convert the argument to --device= to its internal format. """
        # --device can't just take an int in the callback above, because it
        # could be specificed as "mdX", which causes optparse to error when
        # it runs int().
        return int(s)

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        if any(arg for arg in extra if arg.startswith("-")):
            mapping = {"command": "raid", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        # because positional argumnets with variable number of values
        # don't parse very well
        if not ns.partitions and extra:
            ns.partitions = extra
            extra = []

        if not ns.format:
            ns.preexist = True

        assert len(ns.mntpoint) == 1
        if not ns.partitions and not ns.preexist:
            raise KickstartParseError(_("Partitions required for %s") % "raid", lineno=self.lineno)
        elif ns.partitions and ns.preexist:
            raise KickstartParseError(_("Members may not be specified for preexisting RAID device"), lineno=self.lineno)

        rd = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, rd)
        rd.lineno = self.lineno

        # In older pykickstart --device was always specifying a minor, so
        # rd.device had to be an integer.
        # In newer pykickstart it has to be the array name since the minor
        # cannot be reliably predicted due to lack of mdadm.conf during boot.
        rd.device = self._getDevice(rd.device)
        rd.mountpoint = ns.mntpoint[0]

        if len(ns.partitions) > 0:
            rd.members = ns.partitions

        # Check for duplicates in the data list.
        if rd in self.dataList():
            warnings.warn(_("A RAID device with the name %s has already been defined.") % rd.device, KickstartParseWarning)

        if not rd.preexist and not rd.level:
            raise KickstartParseError("RAID Partition defined without RAID level", lineno=self.lineno)

        if rd.preexist and rd.device == "":
            raise KickstartParseError("Device required for preexisting RAID device", lineno=self.lineno)

        return rd

    def dataList(self):
        return self.raidList

    @property
    def dataClass(self):
        return self.handler.RaidData

class FC4_Raid(FC3_Raid):
    removedKeywords = FC3_Raid.removedKeywords
    removedAttrs = FC3_Raid.removedAttrs

    def _getParser(self):
        op = FC3_Raid._getParser(self)
        op.add_argument("--fsoptions", dest="fsopts", version=FC4, help="""
                        Specifies a free form string of options to be used when
                        mounting the filesystem. This string will be copied into
                        the /etc/fstab file of the installed system and should
                        be enclosed in quotes.""")
        return op

class FC5_Raid(FC4_Raid):
    removedKeywords = FC4_Raid.removedKeywords
    removedAttrs = FC4_Raid.removedAttrs

    def _getParser(self):
        op = FC4_Raid._getParser(self)
        op.add_argument("--bytes-per-inode", dest="bytesPerInode", type=int,
                        version=FC5, help="Specify the bytes/inode ratio.")
        return op

class RHEL5_Raid(FC5_Raid):
    removedKeywords = FC5_Raid.removedKeywords
    removedAttrs = FC5_Raid.removedAttrs

    def __init__(self, writePriority=131, *args, **kwargs):
        FC5_Raid.__init__(self, writePriority, *args, **kwargs)

        self.levelMap.update({"RAID10": "RAID10", "10": "RAID10"})

    def _getParser(self):
        op = FC5_Raid._getParser(self)
        for action in op._actions:
            if "--level" in action.option_strings:
                action.help += dedent("""

                .. versionchanged:: %s

                The "RAID10" level was added.""" % versionToLongString(RHEL5))
                break

        op.add_argument("--encrypted", action="store_true",
                        default=False, version=RHEL5, help="""
                        Specify that this RAID device should be encrypted.
                        """)
        op.add_argument("--passphrase", version=RHEL5, help="""
                        Specify the passphrase to use when encrypting this RAID
                        device. Without the above --encrypted option, this
                        option does nothing. If no passphrase is specified, the
                        default system-wide one is used, or the installer will
                        stop and prompt if there is no default.""")
        return op

class F7_Raid(FC5_Raid):
    removedKeywords = FC5_Raid.removedKeywords
    removedAttrs = FC5_Raid.removedAttrs

    def __init__(self, writePriority=131, *args, **kwargs):
        FC5_Raid.__init__(self, writePriority, *args, **kwargs)

        self.levelMap.update({"RAID10": "RAID10", "10": "RAID10"})

    def _getParser(self):
        op = FC5_Raid._getParser(self)
        for action in op._actions:
            if "--level" in action.option_strings:
                action.help += dedent("""

                .. versionchanged:: %s

                The "RAID10" level was added.""" % versionToLongString(F7))
                break
        return op

class F9_Raid(F7_Raid):
    removedKeywords = F7_Raid.removedKeywords
    removedAttrs = F7_Raid.removedAttrs

    def _getParser(self):
        op = F7_Raid._getParser(self)
        op.add_argument("--bytes-per-inode", deprecated=F9)
        op.add_argument("--fsprofile", version=F9, help="""
                        Specifies a usage type to be passed to the program that
                        makes a filesystem on this partition. A usage type
                        defines a variety of tuning parameters to be used when
                        making a filesystem. For this option to work, the
                        filesystem must support the concept of usage types and
                        there must be a configuration file that lists valid
                        types. For ext2/3/4, this configuration file is
                        ``/etc/mke2fs.conf``.""")
        op.add_argument("--encrypted", action="store_true", version=F9,
                        default=False, help="""
                        Specify that this RAID device should be encrypted.""")
        op.add_argument("--passphrase", version=F9, help="""
                        Specify the passphrase to use when encrypting this RAID
                        device. Without the above --encrypted option, this option
                        does nothing. If no passphrase is specified, the default
                        system-wide one is used, or the installer will stop and
                        prompt if there is no default.""")
        return op

class F12_Raid(F9_Raid):
    removedKeywords = F9_Raid.removedKeywords
    removedAttrs = F9_Raid.removedAttrs

    def _getParser(self):
        op = F9_Raid._getParser(self)
        op.add_argument("--escrowcert", metavar="<url>", version=F12, help="""
                        Load an X.509 certificate from ``<url>``. Store the
                        data encryption key of this partition, encrypted using
                        the certificate, as a file in ``/root``. Only relevant
                        if ``--encrypted`` is specified as well.""")
        op.add_argument("--backuppassphrase", action="store_true",
                        default=False, version=F12, help="""
                        Only relevant if ``--escrowcert`` is specified as well.
                        In addition to storing the data encryption key, generate
                        a random passphrase and add it to this partition. Then
                        store the passphrase, encrypted using the certificate
                        specified by ``--escrowcert``, as a file in ``/root``.
                        If more than one LUKS volume uses ``--backuppassphrase``,
                        the same passphrase will be used for all such volumes.
                        """)
        return op

class F13_Raid(F12_Raid):
    removedKeywords = F12_Raid.removedKeywords
    removedAttrs = F12_Raid.removedAttrs

    def __init__(self, writePriority=131, *args, **kwargs):
        F12_Raid.__init__(self, writePriority, *args, **kwargs)

        self.levelMap.update({"RAID4": "RAID4", "4": "RAID4"})

    def _getParser(self):
        op = F12_Raid._getParser(self)
        for action in op._actions:
            if "--level" in action.option_strings:
                action.help += dedent("""

                .. versionchanged:: %s

                The "RAID4" level was added.""" % versionToLongString(F13))
                break
        return op

class RHEL6_Raid(F13_Raid):
    removedKeywords = F13_Raid.removedKeywords
    removedAttrs = F13_Raid.removedAttrs

    def _getParser(self):
        op = F13_Raid._getParser(self)
        op.add_argument("--cipher", version=RHEL6, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        return op

    def parse(self, args):
        # first call the overriden method
        retval = F13_Raid.parse(self, args)
        # the raid command can't be used together with the autopart command
        # due to the hard to debug behavior their combination introduces
        if self.handler.autopart.seen:
            errorMsg = _("The raid and autopart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        return retval

class F14_Raid(F13_Raid):
    removedKeywords = F13_Raid.removedKeywords
    removedAttrs = F13_Raid.removedAttrs

    def _getParser(self):
        op = F13_Raid._getParser(self)
        op.remove_argument("--bytes-per-inode", version=F14)
        return op

class F15_Raid(F14_Raid):
    removedKeywords = F14_Raid.removedKeywords
    removedAttrs = F14_Raid.removedAttrs

    def _getParser(self):
        op = F14_Raid._getParser(self)
        op.add_argument("--label", version=F15, help="""
                        Specify the label to give to the filesystem to be made.
                        If the given label is already in use by another
                        filesystem, a new label will be created.""")
        return op

class F18_Raid(F15_Raid):
    removedKeywords = F15_Raid.removedKeywords
    removedAttrs = F15_Raid.removedAttrs

    def _getParser(self):
        op = F15_Raid._getParser(self)
        op.add_argument("--cipher", version=F18, help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which encryption algorithm should be used to encrypt the
                        filesystem.""")
        return op

class F19_Raid(F18_Raid):
    removedKeywords = F18_Raid.removedKeywords
    removedAttrs = F18_Raid.removedAttrs

    def _getDevice(self, s):
        return s

class F20_Raid(F19_Raid):
    removedKeywords = F19_Raid.removedKeywords
    removedAttrs = F19_Raid.removedAttrs

    def parse(self, args):
        # first call the overriden method
        retval = F19_Raid.parse(self, args)
        # the raid command can't be used together with the autopart command
        # due to the hard to debug behavior their combination introduces
        if self.handler.autopart.seen:
            errorMsg = _("The raid and autopart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        # the same applies to the 'mount' command
        if hasattr(self.handler, "mount") and self.handler.mount.seen:
            errorMsg = _("The raid and mount commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        return retval

class F23_Raid(F20_Raid):
    removedKeywords = F20_Raid.removedKeywords
    removedAttrs = F20_Raid.removedAttrs

    def _getParser(self):
        op = F20_Raid._getParser(self)
        op.add_argument("--mkfsoptions", dest="mkfsopts", version=F23, help="""
                        Specifies additional parameters to be passed to the
                        program that makes a filesystem on this partition. No
                        processing is done on the list of arguments, so they
                        must be supplied in a format that can be passed directly
                        to the mkfs program. This means multiple options should
                        be comma-separated or surrounded by double quotes,
                        depending on the filesystem.""")
        return op

    def parse(self, args):
        retval = F20_Raid.parse(self, args)

        if not retval.format and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions with --noformat has no effect."), lineno=self.lineno)

        if retval.fsprofile and retval.mkfsopts:
            raise KickstartParseError(_("--mkfsoptions and --fsprofile cannot be used together."), lineno=self.lineno)

        return retval

class F25_Raid(F23_Raid):
    removedKeywords = F23_Raid.removedKeywords
    removedAttrs = F23_Raid.removedAttrs

    def _getParser(self):
        op = F23_Raid._getParser(self)
        op.add_argument("--chunksize", type=int, dest="chunk_size",
                        version=F25, help="""
                        Specify the chunk size (in KiB) for this RAID array.
                        """)
        return op

class RHEL7_Raid(F25_Raid):
    pass

class F29_Raid(F25_Raid):
    removedKeywords = F25_Raid.removedKeywords
    removedAttrs = F25_Raid.removedAttrs

    def _getParser(self):
        op = F25_Raid._getParser(self)
        op.add_argument("--luks-version", dest="luks_version", version=F29, default="",
                        help="""
                        Only relevant if ``--encrypted`` is specified. Specifies
                        which version of LUKS format should be used to encrypt
                        the filesystem.""")
        op.add_argument("--pbkdf", version=F29, default="", help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        Password-Based Key Derivation Function (PBKDF) algorithm
                        for LUKS keyslot. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-memory", dest="pbkdf_memory", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the memory cost for PBKDF. See ``man cryptsetup``.""")
        op.add_argument("--pbkdf-time", dest="pbkdf_time", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of milliseconds to spend with PBKDF passphrase
                        processing. See ``--iter-time`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        op.add_argument("--pbkdf-iterations", dest="pbkdf_iterations", type=int, default=0,
                        version=F29, help="""
                        Only relevant if ``--encrypted`` is specified. Sets
                        the number of iterations directly and avoids PBKDF benchmark.
                        See ``--pbkdf-force-iterations`` in ``man cryptsetup``.

                        Only one of ``--pbkdf-time`` and ``--pbkdf-iterations``
                        can be specified.
                        """)
        return op

    def parse(self, args):
        retval = F25_Raid.parse(self, args)

        if retval.pbkdf_time and retval.pbkdf_iterations:
            msg = _("Only one of --pbkdf-time and --pbkdf-iterations can be specified.")
            raise KickstartParseError(msg, lineno=self.lineno)

        return retval

class RHEL8_Raid(F29_Raid):
    removedKeywords = F29_Raid.removedKeywords
    removedAttrs = F29_Raid.removedAttrs

    def parse(self, args):
        retval = F29_Raid.parse(self, args)
        if retval.fstype == "btrfs":
            raise KickstartParseError(_("Btrfs file system is not supported"), lineno=self.lineno)
        return retval

    def _getParser(self):
        "Only necessary for the type change documentation"
        op = F29_Raid._getParser(self)
        for action in op._actions:
            if "--fstype" in action.option_strings:
                action.help += """

                    .. versionchanged:: %s

                    Btrfs support was removed.""" % versionToLongString(RHEL8)
        return op



#
# Radek Vykydal <rvykydal@redhat.com>
#
# Copyright 2018 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import warnings
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser, commaSplit
from pykickstart.constants import NVDIMM_MODE_SECTOR, NVDIMM_ACTION_RECONFIGURE, \
    NVDIMM_ACTION_USE
from pykickstart.version import F28

from pykickstart.i18n import _


class F28_NvdimmData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.action = kwargs.get("action", None)
        self.namespace = kwargs.get("namespace", "")
        self.mode = kwargs.get("mode", None)
        self.sectorsize = kwargs.get("sectorsize", None)
        self.blockdevs = kwargs.get("blockdevs", [])

    def __eq__(self, y):
        if not y:
            return False
        return (self.action == y.action and
                self.namespace == y.namespace and
                self.blockdevs == y.blockdevs)

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = "%s" % self.action
        if self.action == NVDIMM_ACTION_RECONFIGURE:
            if self.namespace:
                retval += " --namespace=%s" % self.namespace
            if self.mode == NVDIMM_MODE_SECTOR:
                retval += " --mode=%s" % NVDIMM_MODE_SECTOR
                if self.sectorsize:
                    retval += " --sectorsize=%d" % self.sectorsize
        elif self.action == NVDIMM_ACTION_USE:
            if self.namespace:
                retval += " --namespace=%s" % self.namespace
            if self.blockdevs:
                retval += " --blockdevs=%s" % ",".join(self.blockdevs)

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "nvdimm %s" % self._getArgsAsStr()
        return retval.strip() + "\n"


class F28_Nvdimm(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=80, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

        self.actionList = kwargs.get("actionList", [])
        self.validActions = [NVDIMM_ACTION_RECONFIGURE, NVDIMM_ACTION_USE]
        self.validModes = [NVDIMM_MODE_SECTOR]

        self.op = self._getParser()

    def __str__(self):
        retval = ""

        for action in self.actionList:
            retval += action.__str__()

        if retval != "":
            retval = "# NVDIMM devices setup\n" + retval + "\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="nvdimm", version=F28, description="""
                            Perform an action on an nvdimm device.""")
        op.add_argument("action", choices=self.validActions, nargs=1, default=NVDIMM_ACTION_RECONFIGURE,
                        version=F28, help="""
                        The action to be performed on the device specified by further options.
                        The device can be specified by ``--namespace`` or ``--blockdevs`` options,
                        depending on the action.

                        Valid actions:
                        - ``reconfigure``: Reconfigures the device specified by ``--namespace``
                                           into the mode specified by ``--mode`` and (depending on the mode)
                                           ``--sectorsize`` options. The device reconfigured into sector mode
                                           will be allowed to be used for storage configuration.
                        - ``use``: Allow the device to be used for storage configuration. By default nvdimm
                                   devices are ignored. Only devices in sector mode can be used.
                        """)
        op.add_argument("--namespace", metavar="<namespace>", version=F28,
                        help="""The device specification by namespace.""")
        op.add_argument("--blockdevs", metavar="<devspec1>,<devspec2>,...,<devspecN>", type=commaSplit, version=F28,
                        help="""Specification of devices by comma separated list of block device names.""")
        op.add_argument("--mode", choices=self.validModes, default=NVDIMM_MODE_SECTOR,
                        version=F28, help="""The mode specification.""")
        op.add_argument("--sectorsize", type=int, version=F28,
                        help="""Size of a sector for sector mode.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if extra:
            raise KickstartParseError(_("Unexpected arguments to nvdimm command: %(arguments)s") %
                                      {"arguments": extra}, lineno=self.lineno)

        nvdimm_data = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, nvdimm_data)
        nvdimm_data.lineno = self.lineno
        action = ns.action[0]
        nvdimm_data.action = action

        if nvdimm_data.namespace and nvdimm_data.blockdevs:
            raise KickstartParseError(_("Only one of --namespace and --blockdevs device specifications can be used"))

        # Check for duplicates in the data list.
        if nvdimm_data in self.dataList():
            if nvdimm_data.namespace:
                warnings.warn(_("An action %(action)s on namespace %(namespace)s has already been defined.")
                              % {"action": action, "namespace": nvdimm_data.namespace}, KickstartParseWarning)
            if nvdimm_data.blockdevs:
                warnings.warn(_("An action %(action)s on devices %(blockdevs)s has already been defined.")
                              % {"action": action, "blockdevs": nvdimm_data.blockdevs}, KickstartParseWarning)

        if action == NVDIMM_ACTION_RECONFIGURE:
            if not nvdimm_data.namespace:
                raise KickstartParseError(_("Action %(action)s requires --namespace argument to be set")
                                          % {"action": action})
            if nvdimm_data.mode == NVDIMM_MODE_SECTOR and not nvdimm_data.sectorsize:
                raise KickstartParseError(_("Action %(action)s with mode %(mode)s requires --sectorsize argument to be set")
                                          % {"action": action, "mode": nvdimm_data.mode})
        elif action == NVDIMM_ACTION_USE:
            if not nvdimm_data.namespace and not nvdimm_data.blockdevs:
                raise KickstartParseError(_("Action %(action)s requires --namespace or --blockdevs argument to be set")
                                          % {"action": action})

        return nvdimm_data

    def dataList(self):
        return self.actionList

    @property
    def dataClass(self):
        return self.handler.NvdimmData



import unittest
from tests.baseclass import CommandTest
from pykickstart.commands.vnc import FC3_Vnc

class Vnc_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_Vnc()
        self.assertEqual(cmd.enabled, False)

class FC3_TestCase(CommandTest):
    command = "vnc"

    def runTest(self):
        obj = self.assert_parse("vnc", "vnc\n")
        obj.enabled = False
        self.assertEqual(str(obj), "")

        # pass
        self.assert_parse("vnc --connect=HOSTNAME", "vnc --connect=HOSTNAME\n")
        self.assert_parse("vnc --connect=HOSTNAME:PORT", "vnc --connect=HOSTNAME:PORT\n")
        self.assert_parse("vnc --password=PASSWORD", "vnc --password=PASSWORD\n")
        self.assert_parse("vnc --connect=HOSTNAME --password=PASSWORD", "vnc --connect=HOSTNAME --password=PASSWORD\n")

        # fail
        self.assert_parse_error("vnc --connect")
        self.assert_parse_error("vnc --password")

class FC6_TestCase(CommandTest):
    command = "vnc"

    def runTest(self):
        # pass
        self.assert_parse("vnc", "vnc\n")
        self.assert_parse("vnc --host=HOSTNAME", "vnc --host=HOSTNAME\n")
        self.assert_parse("vnc --port=PORT", "vnc\n")
        self.assert_parse("vnc --password=PASSWORD", "vnc --password=PASSWORD\n")

        if "--connect" in self.optionList:
            self.assert_parse("vnc --connect=HOSTNAME", "vnc --host=HOSTNAME\n")
            self.assert_parse("vnc --connect=HOSTNAME:PORT", "vnc --host=HOSTNAME --port=PORT\n")
            self.assert_parse("vnc --connect=HOSTNAME --password=PASSWORD", "vnc --host=HOSTNAME --password=PASSWORD\n")
            self.assert_parse("vnc --connect=HOSTNAME:PORT --password=PASSWORD", "vnc --host=HOSTNAME --port=PORT --password=PASSWORD\n")

        # Ensure --connect has been deprecated
        self.assert_deprecated("vnc", "connect")

        # fail
        self.assert_parse_error("vnc --connect")
        self.assert_parse_error("vnc --password")

class F9_TestCase(FC6_TestCase):
    def runTest(self):
        FC6_TestCase.runTest(self)

        # Ensure --connect has been removed
        self.assert_removed("vnc", "connect")

        # Any --connect use should raise KickstartParseError
        self.assert_parse_error("vnc --host=HOSTNAME --connect=HOSTNAME --password=PASSWORD")
        self.assert_parse_error("vnc --host=HOSTNAME --connect=HOSTNAME --password=PASSWORD")
        self.assert_parse_error("vnc --connect=HOSTNAME --password=PASSWORD")
        self.assert_parse_error("vnc --connect=HOSTNAME")
        self.assert_parse_error("vnc --connect")
        self.assert_parse_error("vnc --password")

if __name__ == "__main__":
    unittest.main()



#
# Copyright 2019 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from tests.baseclass import CommandTest


class RHEL8_TestCase(CommandTest):
    command = "zipl"

    def runTest(self):
        self.assert_parse("zipl", "")
        self.assert_parse("zipl --secure-boot", "zipl --secure-boot\n")
        self.assert_parse("zipl --force-secure-boot", "zipl --force-secure-boot\n")
        self.assert_parse("zipl --no-secure-boot", "zipl --no-secure-boot\n")

        self.assert_parse_error("zipl --invalid")
        self.assert_parse_error("zipl --secure-boot=")
        self.assert_parse_error("zipl --no-secure-boot=")
        self.assert_parse_error("zipl --force-secure-boot=")


if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2007, 2008, 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent
from pykickstart.version import versionToLongString
from pykickstart.version import FC6, F8, F11, F13, F14, F15, F21, F27, RHEL8
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartError, KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser, commaSplit, ksboolean

import warnings
from pykickstart.i18n import _

class FC6_RepoData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.baseurl = kwargs.get("baseurl", "")
        self.mirrorlist = kwargs.get("mirrorlist", None)
        self.name = kwargs.get("name", "")

    def __eq__(self, y):
        if not y:
            return False

        return self.name == y.name

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""

        if self.baseurl:
            retval += "--baseurl=%s" % self.baseurl
        elif self.mirrorlist:
            retval += "--mirrorlist=%s" % self.mirrorlist

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "repo --name=\"%s\" %s\n" % (self.name, self._getArgsAsStr())
        return retval

class F8_RepoData(FC6_RepoData):
    removedKeywords = FC6_RepoData.removedKeywords
    removedAttrs = FC6_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        FC6_RepoData.__init__(self, *args, **kwargs)
        self.cost = kwargs.get("cost", None)
        self.includepkgs = kwargs.get("includepkgs", [])
        self.excludepkgs = kwargs.get("excludepkgs", [])

    def _getArgsAsStr(self):
        retval = FC6_RepoData._getArgsAsStr(self)

        if self.cost:
            retval += " --cost=%s" % self.cost
        if self.includepkgs:
            retval += " --includepkgs=\"%s\"" % ",".join(self.includepkgs)
        if self.excludepkgs:
            retval += " --excludepkgs=\"%s\"" % ",".join(self.excludepkgs)

        return retval

class F11_RepoData(F8_RepoData):
    removedKeywords = F8_RepoData.removedKeywords
    removedAttrs = F8_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F8_RepoData.__init__(self, *args, **kwargs)
        self.ignoregroups = kwargs.get("ignoregroups", None)

    def _getArgsAsStr(self):
        retval = F8_RepoData._getArgsAsStr(self)

        if self.ignoregroups:
            retval += " --ignoregroups=true"
        return retval

class F13_RepoData(F11_RepoData):
    removedKeywords = F11_RepoData.removedKeywords
    removedAttrs = F11_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F11_RepoData.__init__(self, *args, **kwargs)
        self.proxy = kwargs.get("proxy", "")

    def _getArgsAsStr(self):
        retval = F11_RepoData._getArgsAsStr(self)

        if self.proxy:
            retval += " --proxy=\"%s\"" % self.proxy

        return retval

class F14_RepoData(F13_RepoData):
    removedKeywords = F13_RepoData.removedKeywords
    removedAttrs = F13_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_RepoData.__init__(self, *args, **kwargs)
        self.noverifyssl = kwargs.get("noverifyssl", False)

    def _getArgsAsStr(self):
        retval = F13_RepoData._getArgsAsStr(self)

        if self.noverifyssl:
            retval += " --noverifyssl"

        return retval

class RHEL6_RepoData(F14_RepoData):
    pass

class F15_RepoData(F14_RepoData):
    pass

class F21_RepoData(F15_RepoData):
    removedKeywords = F15_RepoData.removedKeywords
    removedAttrs = F15_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F15_RepoData.__init__(self, *args, **kwargs)
        self.install = kwargs.get("install", False)

    def _getArgsAsStr(self):
        retval = F15_RepoData._getArgsAsStr(self)

        if self.install:
            retval += " --install"

        return retval

class F27_RepoData(F21_RepoData):
    removedKeywords = F21_RepoData.removedKeywords
    removedAttrs = F21_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F21_RepoData.__init__(self, *args, **kwargs)
        self.metalink = kwargs.get("metalink", False)

    def _getArgsAsStr(self):
        retval = F21_RepoData._getArgsAsStr(self)

        if self.metalink:
            retval += " --metalink=%s" % self.metalink

        return retval

class RHEL8_RepoData(F27_RepoData):
    removedKeywords = F27_RepoData.removedKeywords
    removedAttrs = F27_RepoData.removedAttrs

    def __init__(self, *args, **kwargs):
        F27_RepoData.__init__(self, *args, **kwargs)
        self.sslcacert = kwargs.get("sslcacert", None)
        self.sslclientcert = kwargs.get("sslclientcert", None)
        self.sslclientkey = kwargs.get("sslclientkey", None)

    def _getArgsAsStr(self):
        retval = F27_RepoData._getArgsAsStr(self)

        if self.sslcacert:
            retval += " --sslcacert=\"%s\"" % self.sslcacert

        if self.sslclientcert:
            retval += " --sslclientcert=\"%s\"" % self.sslclientcert

        if self.sslclientkey:
            retval += " --sslclientkey=\"%s\"" % self.sslclientkey

        return retval

class RHEL7_RepoData(F21_RepoData):
    pass

class FC6_Repo(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    urlRequired = True

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.repoList = kwargs.get("repoList", [])
        self.exclusive_required_options = [("mirrorlist", "--mirrorlist"),
                                           ("baseurl", "--baseurl")]

    def __str__(self):
        retval = ""
        for repo in self.repoList:
            retval += repo.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="repo", description="""
                            Configures additional yum repositories that may be
                            used as sources for package installation. Multiple
                            repo lines may be specified. By default, anaconda
                            has a configured set of repos taken from
                            /etc/anaconda.repos.d plus a special Installation
                            Repo in the case of a media install. The exact set
                            of repos in this directory changes from release to
                            release and cannot be listed here. There will
                            likely always be a repo named "updates".

                            Note: If you want to enable one of the repos in
                            /etc/anaconda.repos.d that is disabled by default
                            (like "updates"), you should use --name= but none
                            of the other options. anaconda will look for a repo
                            by this name automatically. Providing a baseurl or
                            mirrorlist URL will result in anaconda attempting
                            to add another repo by the same name, which will
                            cause a conflicting repo error.""",
                            version=FC6)
        op.add_argument("--name", required=True, version=FC6, help="""
                        The repo id. This option is required. The RepoId must 
                        not contain spaces (do not confuse with the optional name
                        used by yum). If a repo has a name that conflicts with a 
                        previously added one, the new repo will be ignored. 
                        Because anaconda has a populated list of repos when it 
                        starts, this means that users cannot create new repos 
                        that override these names.
                        Please check /etc/anaconda.repos.d from the operating
                        system you wish to install to see what names are not
                        available.""")
        op.add_argument("--baseurl", version=FC6, help="""
                        The URL for the repository. The variables that may be
                        used in yum repo config files are not supported here.
                        You may use one of either this option or
                        ``--mirrorlist``, not both. If an NFS repository is
                        specified, it should be of the form
                        ``nfs://host:/path/to/repo``. Note that there is a
                        colon after the host. Anaconda passes everything after
                        "nfs:// " directly to the mount command instead of
                        parsing URLs according to RFC 2224. Variable
                        substitution is done for $releasever and $basearch in
                        the url.""")
        op.add_argument("--mirrorlist", version=FC6, help="""
                        The URL pointing at a list of mirrors for the
                        repository. The variables that may be used in yum repo
                        config files are not supported here. You may use one of
                        either this option or ``--baseurl``, not both. Variable
                        substitution is done for $releasever and $basearch in
                        the url.""")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)

        # Check that just one of exclusive required options is specified
        used_options = [opt for attr, opt in self.exclusive_required_options
                        if getattr(ns, attr, None)]
        if self.urlRequired and not used_options:
            mapping = {"options_list": ", ".join((opt for attr, opt in self.exclusive_required_options))}
            raise KickstartParseError(_("One of -%(options_list)s options must be specified for repo command.") % mapping, lineno=self.lineno)
        if len(used_options) > 1:
            mapping = {"options_list": ", ".join((opt for opt in used_options))}
            raise KickstartParseError(_("Only one of %(options_list)s options may be specified for repo command.") % mapping, lineno=self.lineno)

        rd = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, rd)
        rd.lineno = self.lineno

        # Check for duplicates in the data list.
        if rd in self.dataList():
            warnings.warn(_("A repo with the name %s has already been defined.") % rd.name, KickstartParseWarning)

        return rd

    def dataList(self):
        return self.repoList

    @property
    def dataClass(self):
        return self.handler.RepoData

class F8_Repo(FC6_Repo):
    removedKeywords = FC6_Repo.removedKeywords
    removedAttrs = FC6_Repo.removedAttrs

    def __str__(self):
        retval = ""
        for repo in self.repoList:
            retval += repo.__str__()

        return retval

    def _getParser(self):
        op = FC6_Repo._getParser(self)
        op.add_argument("--cost", type=int, version=F8, help="""
                        An integer value to assign a cost to this repository.
                        If multiple repositories provide the same packages,
                        this number will be used to prioritize which repository
                        will be used before another. Repositories with a lower
                        cost take priority over repositories with higher cost.
                        """)
        op.add_argument("--excludepkgs", type=commaSplit, version=F8, help="""
                        A comma-separated list of package names and globs that
                        must not be pulled from this repository. This is useful
                        if multiple repositories provide the same package and
                        you want to make sure it comes from a particular
                        repository.""")
        op.add_argument("--includepkgs", type=commaSplit, version=F8, help="""
                        A comma-separated list of package names and globs that
                        must be pulled from this repository. This is useful if
                        multiple repositories provide the same package and you
                        want to make sure it comes from this repository.""")
        return op

    def methodToRepo(self):
        if not self.handler.method.url:
            raise KickstartError(_("Method must be a url to be added to the repo list."), lineno=self.handler.method.lineno)
        reponame = "ks-method-url"
        repourl = self.handler.method.url
        rd = self.handler.RepoData(name=reponame, baseurl=repourl)
        return rd

class F11_Repo(F8_Repo):
    removedKeywords = F8_Repo.removedKeywords
    removedAttrs = F8_Repo.removedAttrs

    def _getParser(self):
        op = F8_Repo._getParser(self)
        op.add_argument("--ignoregroups", type=ksboolean, version=F11, help="""
                        This option is used when composing installation trees
                        and has no effect on the installation process itself.
                        It tells the compose tools to not look at the package
                        group information when mirroring trees so as to avoid
                        mirroring large amounts of unnecessary data.""")
        return op

class F13_Repo(F11_Repo):
    removedKeywords = F11_Repo.removedKeywords
    removedAttrs = F11_Repo.removedAttrs

    def _getParser(self):
        op = F11_Repo._getParser(self)
        op.add_argument("--proxy", version=F13, help="""
                        Specify an HTTP/HTTPS/FTP proxy to use just for this
                        repository. This setting does not affect any other
                        repositories, nor how the install.img is fetched on
                        HTTP installs. The various parts of the argument act
                        like you would expect. The syntax is::

                        ``--proxy=[protocol://][username[:password]@]host[:port]``
                        """)
        return op

class F14_Repo(F13_Repo):
    removedKeywords = F13_Repo.removedKeywords
    removedAttrs = F13_Repo.removedAttrs

    def _getParser(self):
        op = F13_Repo._getParser(self)
        op.add_argument("--noverifyssl", action="store_true", version=F14,
                        default=False, help="""
                        For a https repo do not check the server's certificate
                        with what well-known CA validate and do not check the
                        server's hostname matches the certificate's domain name.
                        """)
        return op

class RHEL6_Repo(F14_Repo):
    pass

class F15_Repo(F14_Repo):
    removedKeywords = F14_Repo.removedKeywords
    removedAttrs = F14_Repo.removedAttrs

    urlRequired = False

    def _getParser(self):
        op = F14_Repo._getParser(self)
        for action in op._actions:
            for option in ['--baseurl', '--mirrorlist']:
                if option in action.option_strings:
                    action.help += dedent("""

                    .. versionchanged:: %s

                    ``--mirrorlist`` and ``--baseurl`` are not required anymore!
                    """ % versionToLongString(F15))
        return op

class F21_Repo(F15_Repo):
    removedKeywords = F15_Repo.removedKeywords
    removedAttrs = F15_Repo.removedAttrs

    def _getParser(self):
        op = F15_Repo._getParser(self)
        op.add_argument("--install", action="store_true", version=F21,
                        default=False, help="""
                        Install this repository to the target system so that it
                        can be used after reboot.""")
        return op

class F27_Repo(F21_Repo):
    removedKeywords = F21_Repo.removedKeywords
    removedAttrs = F21_Repo.removedAttrs

    def __init__(self, *args, **kwargs):
        F21_Repo.__init__(self, *args, **kwargs)
        self.exclusive_required_options.append(("metalink", "--metalink"))

    def _getParser(self):
        op = F21_Repo._getParser(self)
        op.add_argument("--metalink", version=F27, help="""
                        The URL pointing at a metalink for the repository. The
                        variables that may be used in yum repo config files are
                        not supported here. You may use only one of the
                        ``--baseurl``, ``--mirrorlist``, or ``--metalink``
                        options. Variable substitution is done for $releasever
                        and $basearch in the url.""")
        for action in op._actions:
            for option in ['--baseurl', '--mirrorlist']:
                if option in action.option_strings:
                    action.help += dedent("""

                    .. versionchanged:: %s

                    ``Another mutually exclusive option --metalink`` was added.
                    """ % versionToLongString(F27))
        return op

class RHEL8_Repo(F27_Repo):
    removedKeywords = F27_Repo.removedKeywords
    removedAttrs = F27_Repo.removedAttrs

    def _getParser(self):
        op = F27_Repo._getParser(self)
        op.add_argument("--sslcacert", version=RHEL8, help="""
                        Path to the file holding one or more SSL certificates
                        to verify the repository host with.""")
        op.add_argument("--sslclientcert", version=RHEL8, help="""
                        Path to the SSL client certificate (PEM file) which
                        should be used to connect to the repository.""")
        op.add_argument("--sslclientkey", version=RHEL8, help="""
                        Path to the private key file associated with the client
                        certificate given with --sslclientcert.""")
        return op

class RHEL7_Repo(F21_Repo):
    pass



#
# James Laska <jlaska@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest

from pykickstart.errors import KickstartParseWarning
from tests.baseclass import CommandTest

from pykickstart.base import DeprecatedCommand

class FC3_TestCase(CommandTest):
    command = "device"

    def runTest(self):
        # pass
        self.assert_parse("device TYPE MODNAME", "device TYPE MODNAME\n")
        self.assert_parse("device TYPE MODNAME --opts=key1",
                          "device TYPE MODNAME --opts=\"key1\"\n")
        self.assert_parse("device TYPE MODNAME --opts=key1=val1",
                          "device TYPE MODNAME --opts=\"key1=val1\"\n")
        self.assert_parse("device TYPE MODNAME --opts=\"key1=val1\"",
                          "device TYPE MODNAME --opts=\"key1=val1\"\n")
        self.assert_parse("device TYPE MODNAME --opts=\"key1=val1 key2=val2\"",
                          "device TYPE MODNAME --opts=\"key1=val1 key2=val2\"\n")

        # fail
        self.assert_parse_error("device")
        self.assert_parse_error("device MODNAME")
        self.assert_parse_error("device TYPE MODNAME GARBAGE")
        self.assert_parse_error("device --opts=foo")
        self.assert_parse_error("device --opts=\"foo\"")
        self.assert_parse_error("device MODNAME --bogus-option")

        # extra test coverage
        device = self.handler().commands["device"]
        device = device.parse(["TYPE", "MODNAME"])
        self.assertFalse(device == "")
        self.assertTrue(device == device)
        self.assertTrue(device != "")
        device.moduleName = ""
        self.assertEqual(device.__str__(), "\n")

class F8_TestCase(CommandTest):
    command = "device"

    def runTest(self):
        # pass
        self.assert_parse("device MODNAME", "device MODNAME\n")
        self.assert_parse("device MODNAME --opts=key1",
                          "device MODNAME --opts=\"key1\"\n")
        self.assert_parse("device MODNAME --opts=key1=val1",
                          "device MODNAME --opts=\"key1=val1\"\n")
        self.assert_parse("device MODNAME --opts=\"key1=val1\"",
                          "device MODNAME --opts=\"key1=val1\"\n")
        self.assert_parse("device MODNAME --opts=\"key1=val1 key2=val2\"",
                          "device MODNAME --opts=\"key1=val1 key2=val2\"\n")

        # fail - TYPE is no longer accepted
        self.assert_parse_error("device TYPE MODNAME")
        self.assert_parse_error("device TYPE MODNAME --opts=\"foo\"")

        # fail
        self.assert_parse_error("device")
        self.assert_parse_error("device MODNAME GARBAGE")
        self.assert_parse_error("device --opts=foo")
        self.assert_parse_error("device --opts=\"foo\"")
        self.assert_parse_error("device --bogus-option")

        # extra test coverage
        device = self.handler().commands["device"]
        pd = device.parse(["MODNAME"])
        self.assertFalse(pd == "")
        self.assertTrue(pd == pd)
        self.assertTrue(pd != "")

        # test if string representation is as expected
        device.deviceList.append(pd)
        self.assertEqual(device.__str__(), "device MODNAME\n")

        # test if trying to define the same module again will raise
        # a warning
        with self.assertRaises(KickstartParseWarning):
            device.parse(["MODNAME"])

        dd = self.handler().DeviceData()
        dd.moduleName = ""
        self.assertEqual(dd.__str__(), "\n")

class F24_TestCase(F8_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("device")
        self.assertEqual(issubclass(parser.__class__, DeprecatedCommand), True)

if __name__ == "__main__":
    unittest.main()



#
# Tomas Radej <tradej@redhat.com>
#
# Copyright 2015 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import requests
import shutil
import six

from pykickstart.errors import KickstartError
from pykickstart.i18n import _
from requests.exceptions import SSLError, RequestException

_is_url = lambda location: '://' in location  # RFC 3986

SSL_VERIFY = True

def load_to_str(location):
    '''Load a destination URL or file into a string.
    Type of input is inferred automatically.

    Arguments:
    location -- URL or file name to load

    Returns: string with contents
    Raises: KickstartError on error reading'''

    if _is_url(location):
        return _load_url(location)
    else:
        return _load_file(location)

def load_to_file(location, destination):
    '''Load a destination URL or file into a file name.
    Type of input is inferred automatically.

    Arguments:
    location -- URL or file name to load
    destination -- destination file name to write to

    Returns: file name with contents
    Raises: KickstartError on error reading or writing'''

    if _is_url(location):
        contents = _load_url(location)

        # Write to file
        try:
            with open(destination, 'w') as fh:
                fh.write(contents)
        except IOError as e:
            raise KickstartError(_('Error writing file "%s":') % location + ': {e}'.format(e=str(e)))

        return destination
    else:
        _copy_file(location, destination)
        return destination

def _load_url(location):
    '''Load a location (URL or filename) and return contents as string'''

    try:
        request = requests.get(location, verify=SSL_VERIFY)
    except SSLError as e:
        raise KickstartError(_('Error securely accessing URL "%s"') % location + ': {e}'.format(e=str(e)))
    except RequestException as e:
        raise KickstartError(_('Error accessing URL "%s"') % location + ': {e}'.format(e=str(e)))

    if request.status_code != requests.codes.ok:        # pylint: disable=no-member
        raise KickstartError(_('Error accessing URL "%s"') % location + ': {c}'.format(c=str(request.status_code)))

    return request.text

def _load_file(filename):
    '''Load a file's contents and return them as a string'''

    try:
        if six.PY3:
            with open(filename, 'rb') as fh:
                contents = fh.read().decode("utf-8")
        else:
            with open(filename, 'r') as fh:
                contents = fh.read()
    except IOError as e:
        raise KickstartError(_('Error opening file: %s') % str(e))

    return contents

def _copy_file(filename, destination):
    '''Copy file to destination'''

    try:
        shutil.copyfile(filename, destination)
    except (OSError, IOError) as e:
        raise KickstartError(_('Error copying file: %s') % str(e))



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from textwrap import dedent
from pykickstart.version import versionToLongString, FC3, FC6, F18, F23
from pykickstart.base import KickstartCommand
from pykickstart.constants import KS_REBOOT, KS_SHUTDOWN, KS_WAIT
from pykickstart.options import KSOptionParser

class FC3_Reboot(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.action = kwargs.get("action", None)

    def _getArgsAsStr(self):
        return ""

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        if self.action == KS_REBOOT:
            retval += "# Reboot after installation\nreboot"
            retval += self._getArgsAsStr() + "\n"
        elif self.action == KS_SHUTDOWN:
            retval += "# Shutdown after installation\nshutdown"
            retval += self._getArgsAsStr() + "\n"

        return retval

    def parse(self, args):
        if self.currentCmd == "reboot":
            self.action = KS_REBOOT
        else:
            self.action = KS_SHUTDOWN

        return self

    def _getParser(self):
        op = KSOptionParser(prog="reboot|poweroff|shutdown", description="""

                            ``reboot``

                            Reboot after the installation is complete. Normally,
                            kickstart displays a message and waits for the user
                            to press a key before rebooting.

                            ``poweroff``

                            Turn off the machine after the installation is complete.
                            Normally, kickstart displays a message and waits for
                            the user to press a key before rebooting.

                            ``shutdown``

                            At the end of installation, shut down the machine.
                            This is the same as the poweroff command. Normally,
                            kickstart displays a message and waits for the user
                            to press a key before rebooting.""", version=FC3)
        return op


class FC6_Reboot(FC3_Reboot):
    removedKeywords = FC3_Reboot.removedKeywords
    removedAttrs = FC3_Reboot.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC3_Reboot.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.eject = kwargs.get("eject", False)

    def _getArgsAsStr(self):
        retval = FC3_Reboot._getArgsAsStr(self)

        if self.eject:
            retval += " --eject"

        return retval

    def _getParser(self):
        op = FC3_Reboot._getParser(self)
        op.add_argument("--eject", action="store_true", version=FC6,
                        default=False, help="""
                        Attempt to eject CD or DVD media before rebooting.
                        """)
        return op

    def parse(self, args):
        FC3_Reboot.parse(self, args)
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        self.set_to_self(ns)
        return self

class F18_Reboot(FC6_Reboot):
    removedKeywords = FC6_Reboot.removedKeywords
    removedAttrs = FC6_Reboot.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        FC6_Reboot.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

    def __str__(self):
        retval = FC6_Reboot.__str__(self)

        if self.action == KS_WAIT:
            retval = "# Halt after installation\nhalt"
            retval += self._getArgsAsStr() + "\n"

        return retval

    def parse(self, args):
        FC6_Reboot.parse(self, args)
        if self.currentCmd == "halt":
            self.action = KS_WAIT
        return self

    def _getParser(self):
        op = FC6_Reboot._getParser(self)
        op.prog += "|halt"
        op.description += dedent("""

        ``halt``

        At the end of installation, display a message and wait for the user to
        press a key before rebooting. This is the default action.

        .. versionchanged:: %s

        The 'halt' command was added!

        """ % versionToLongString(F18))
        return op

class F23_Reboot(F18_Reboot):
    removedKeywords = F18_Reboot.removedKeywords
    removedAttrs = F18_Reboot.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        F18_Reboot.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.kexec = kwargs.get("kexec", False)

    def _getArgsAsStr(self):
        retval = F18_Reboot._getArgsAsStr(self)

        if self.kexec:
            retval += " --kexec"

        return retval

    def _getParser(self):
        op = F18_Reboot._getParser(self)
        op.add_argument("--kexec", action="store_true", version=F23,
                        default=False, help="""
                        Use kexec to reboot into the new system, bypassing
                        BIOS/Firmware and bootloader.""")
        return op



import os
import tempfile

def mktempfile(content="", prefix="ks-", text=True):
    """
    Create a temporary file with defined content and return it's path.
    """
    handle, path = tempfile.mkstemp(prefix=prefix, text=text)
    os.write(handle, str.encode(content))
    os.close(handle)
    return path



#
# Chris Lumens <clumens@redhat.com>
# Peter Jones <pjones@redhat.com>
#
# Copyright 2006, 2007 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import versionToLongString, FC6, F24
from pykickstart.base import BaseData, DeprecatedCommand, KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser

from pykickstart.i18n import _

class FC6_MpPathData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.mpdev = kwargs.get("mpdev", "")
        self.device = kwargs.get("device", "")
        self.rule = kwargs.get("rule", "")
        self.name = ""

    def __str__(self):
        return " --device=%s --rule=\"%s\"" % (self.device, self.rule)

class FC6_MultiPathData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.name = kwargs.get("name", "")
        self.paths = kwargs.get("paths", [])

    def __str__(self):
        retval = BaseData.__str__(self)

        for path in self.paths:
            retval += "multipath --name=%s%s\n" % (self.name, path.__str__())

        return retval

class FC6_MultiPath(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=50, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.mpaths = kwargs.get("mpaths", [])

    def __str__(self):
        retval = ""
        for mpath in self.mpaths:
            retval += mpath.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="multipath", description="", version=FC6)
        op.add_argument("--name", required=True, version=FC6, help="")
        op.add_argument("--device", required=True, notest=True,
                        version=FC6, help="")
        op.add_argument("--rule", required=True, notest=True,
                        version=FC6, help="")
        return op

    def parse(self, args):
        ns = self.op.parse_args(args=args, lineno=self.lineno)
        dd = FC6_MpPathData()
        self.set_to_obj(ns, dd)
        dd.lineno = self.lineno
        dd.mpdev = dd.name.split('/')[-1]

        parent = None
        for x in range(0, len(self.mpaths)):
            mpath = self.mpaths[x]
            for path in mpath.paths:
                if path.device == dd.device:
                    mapping = {"device": path.device, "multipathdev": path.mpdev}
                    raise KickstartParseError(_("Device '%(device)s' is already used in multipath '%(multipathdev)s'") % mapping, lineno=self.lineno)
            if mpath.name == dd.mpdev:
                parent = x

        if not parent:
            mpath = self.dataClass(name=dd.name)    # pylint: disable=not-callable
            mpath.paths.append(dd)
            return mpath
        else:
            mpath = self.mpaths[parent]
            mpath.paths.append(dd)
            return dd

    def dataList(self):
        return self.mpaths

    @property
    def dataClass(self):
        return self.handler.MultiPathData

class F24_MultiPath(DeprecatedCommand, FC6_MultiPath):
    def __init__(self):  # pylint: disable=super-init-not-called
        DeprecatedCommand.__init__(self)

    def _getParser(self):
        op = FC6_MultiPath._getParser(self)
        op.description += "\n\n.. deprecated:: %s" % versionToLongString(F24)
        return op



#
# Peter Jones <pjones@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import F13, F24
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser
import warnings

from pykickstart.i18n import _

class F13_SshPwData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.username = kwargs.get("username", None)
        self.isCrypted = kwargs.get("isCrypted", False)
        self.password = kwargs.get("password", "")
        self.lock = kwargs.get("lock", False)

    def __eq__(self, y):
        if not y:
            return False

        return self.username == y.username

    def __ne__(self, y):
        return not self == y

    def __str__(self):
        retval = BaseData.__str__(self)

        retval += "sshpw"
        retval += self._getArgsAsStr() + '\n'

        return retval

    def _getArgsAsStr(self):
        retval = ""

        retval += " --username=%s" % self.username
        if self.lock:
            retval += " --lock"
        if self.isCrypted:
            retval += " --iscrypted"
        else:
            retval += " --plaintext"

        retval += " %s" % self.password
        return retval

class F24_SshPwData(F13_SshPwData):
    removedKeywords = F13_SshPwData.removedKeywords
    removedAttrs = F13_SshPwData.removedAttrs

    def __init__(self, *args, **kwargs):
        F13_SshPwData.__init__(self, *args, **kwargs)
        self.sshkey = kwargs.get("sshkey", False)

    def _getArgsAsStr(self):
        retval = ""

        retval += " --username=%s" % self.username
        if self.sshkey:
            retval += ' --sshkey'
        else:
            if self.lock:
                retval += " --lock"
            if self.isCrypted:
                retval += " --iscrypted"
            else:
                retval += " --plaintext"

        retval += " %s" % self.password
        return retval

class F13_SshPw(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=0, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.sshUserList = kwargs.get("sshUserList", [])

    def __str__(self):
        retval = ""
        for user in self.sshUserList:
            retval += user.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="sshpw", description="""
                            The installer can start up ssh to provide for
                            interactivity and inspection, just like it can with
                            telnet. The "inst.sshd" option must be specified on
                            the kernel command-line for Anaconda to start an ssh
                            daemon. The sshpw command is used to control the
                            accounts created in the installation environment that
                            may be remotely logged into. For each instance of
                            this command given, a user will be created. These
                            users will not be created on the final system -
                            they only exist for use while the installer is
                            running.

                            Note that by default, root has a blank password. If
                            you don't want any user to be able to ssh in and
                            have full access to your hardware, you must specify
                            sshpw for username root. Also note that if Anaconda
                            fails to parse the kickstart file, it will allow
                            anyone to login as root and have full access to
                            your hardware.""", version=F13)
        op.add_argument("--username", required=True, metavar="<name>",
                        version=F13, help="""
                        Provides the name of the user. This option is required.
                        """)
        op.add_argument("--iscrypted", dest="isCrypted", action="store_true",
                        default=False, version=F13, help="""
                        If this is present, the password argument is assumed to
                        already be encrypted.""")
        op.add_argument("--plaintext", dest="isCrypted", action="store_false",
                        version=F13, help="""
                        If this is present, the password argument is assumed to
                        not be encrypted. This is the default.""")
        op.add_argument("--lock", action="store_true", default=False,
                        version=F13, help="""
                        If this is present, the new user account is locked by
                        default. That is, the user will not be able to login
                        from the console.""")
        op.add_argument("password", metavar="<password>", nargs="*",
                        version=F13, help="""
                        The password string to use.""")
        return op

    def parse(self, args):
        ud = self.dataClass()   # pylint: disable=not-callable
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if not ns.password:
            raise KickstartParseError(_("A single argument is expected for the %s command") % "sshpw", lineno=self.lineno)
        if extra:
            mapping = {"command": "sshpw", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        self.set_to_obj(ns, ud)
        ud.password = " ".join(ns.password)
        ud.lineno = self.lineno

        if ud in self.dataList():
            warnings.warn(_("An ssh user with the name %s has already been defined.") % ud.username, KickstartParseWarning)

        return ud

    def dataList(self):
        return self.sshUserList

    @property
    def dataClass(self):
        return self.handler.SshPwData

class F24_SshPw(F13_SshPw):
    removedKeywords = F13_SshPw.removedKeywords
    removedAttrs = F13_SshPw.removedAttrs

    def _getParser(self):
        op = F13_SshPw._getParser(self)
        op.add_argument("--sshkey", action="store_true", default=False,
                        version=F24, help="""
                        If this is used then the <password> string is
                        interpreted as an ssh key value.""")
        return op



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2005, 2006, 2007, 2012, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.version import FC3, F16, F21
from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError, KickstartParseWarning
from pykickstart.options import KSOptionParser

import warnings
from pykickstart.i18n import _

class FC3_VolGroupData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.format = kwargs.get("format", True)
        self.pesize = kwargs.get("pesize", 32768)
        self.preexist = kwargs.get("preexist", False)
        self.vgname = kwargs.get("vgname", "")
        self.physvols = kwargs.get("physvols", [])

    def __eq__(self, y):
        if not y:
            return False

        return self.vgname == y.vgname

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""
        if not self.format:
            retval += " --noformat"
        if self.pesize:
            retval += " --pesize=%d" % self.pesize
        if self.preexist:
            retval += " --useexisting"

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "volgroup %s" % self.vgname
        retval += self._getArgsAsStr()

        # Do not output the physical volumes list if --preexist was passed in.
        # This would be invalid input according to the parse method.
        if not self.preexist:
            retval += " " + " ".join(self.physvols)

        return retval.strip() + "\n"

class F16_VolGroupData(FC3_VolGroupData):
    def __init__(self, *args, **kwargs):
        FC3_VolGroupData.__init__(self, *args, **kwargs)
        self.reserved_space = kwargs.get("reserved-space", None) or kwargs.get("reserved_space", None)
        self.reserved_percent = kwargs.get("reserved-percent", None) or kwargs.get("reserved_percent", None)

    def _getArgsAsStr(self):
        retval = FC3_VolGroupData._getArgsAsStr(self)
        if self.reserved_space:
            retval += " --reserved-space=%d" % self.reserved_space
        if self.reserved_percent:
            retval += " --reserved-percent=%d" % self.reserved_percent

        return retval

class F21_VolGroupData(F16_VolGroupData):
    def __init__(self, *args, **kwargs):
        F16_VolGroupData.__init__(self, *args, **kwargs)
        self.pesize = kwargs.get("pesize", 0)

class RHEL7_VolGroupData(F21_VolGroupData):
    pass

class RHEL8_VolGroupData(F21_VolGroupData):
    pass

class FC3_VolGroup(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=132, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)
        self.op = self._getParser()

        self.vgList = kwargs.get("vgList", [])

    def __str__(self):
        retval = ""
        for vg in self.vgList:
            retval += vg.__str__()

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="volgroup", description="""
                            Creates a Logical Volume Management (LVM) group.
                            """, epilog="""
                            Create the partition first, create the logical
                            volume group, and then create the logical volume.
                            For example::

                                part pv.01 --size 3000
                                volgroup myvg pv.01
                                logvol / --vgname=myvg --size=2000 --name=rootvol
                            """, version=FC3)
        op.add_argument("name", metavar="<name>", nargs="*", version=FC3, help="""
                        Name given to the volume group. The (which denotes that
                        multiple partitions can be listed) lists the identifiers
                        to add to the volume group.""")
        op.add_argument("partitions", metavar="<partitions*>", nargs="*", help="""
                        Physical Volume partitions to be included in this
                        Volume Group""", version=FC3)
        op.add_argument("--noformat", dest="format", action="store_false",
                        default=True, version=FC3, help="""
                        Use an existing volume group. Do not specify partitions
                        when using this option.""")
        op.add_argument("--pesize", type=int, default=32768, version=FC3,
                        help="""
                        Set the size of the physical extents in KiB.""")
        op.add_argument("--useexisting", dest="preexist", action="store_true",
                        default=False, version=FC3, help="""
                        Use an existing volume group. Do not specify partitions
                        when using this option.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)
        # because positional arguments with variable number of values
        # don't parse very well
        if not ns.partitions:
            if extra:
                ns.partitions = extra
                extra = []
            elif len(ns.name) > 1:
                ns.partitions = ns.name[1:]
                ns.name = [ns.name[0]]

        if not ns.format:
            ns.preexist = True

        vg = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, vg)
        vg.lineno = self.lineno

        if not ns.name:
            raise KickstartParseError(_("volgroup must be given a VG name"), lineno=self.lineno)

        if not any([ns.partitions, ns.preexist]):
            raise KickstartParseError(_("volgroup must be given a list of partitions"), lineno=self.lineno)
        elif ns.partitions and ns.preexist:
            raise KickstartParseError(_("Members may not be specified for preexisting volgroup"), lineno=self.lineno)
        vg.vgname = ns.name[0]

        if ns.partitions:
            vg.physvols = ns.partitions

        # Check for duplicates in the data list.
        if vg in self.dataList():
            warnings.warn(_("A volgroup with the name %s has already been defined.") % vg.vgname, KickstartParseWarning)

        return vg

    def dataList(self):
        return self.vgList

    @property
    def dataClass(self):
        return self.handler.VolGroupData

class F16_VolGroup(FC3_VolGroup):
    def _getParser(self):
        op = FC3_VolGroup._getParser(self)
        op.add_argument("--reserved-space", dest="reserved_space", type=int,
                        version=F16, help="""
                        Specify an amount of space to leave unused in a volume
                        group, in MiB. Do not append any units. This option is
                        only used for new volume groups.""")
        op.add_argument("--reserved-percent", dest="reserved_percent", type=int,
                        version=F16, help="""
                        Specify a percentage of total volume group space to
                        leave unused (new volume groups only).""")
        return op

    def parse(self, args):
        # first call the overriden method
        retval = FC3_VolGroup.parse(self, args)

        # Check that any reserved space options are in their valid ranges.
        if getattr(retval, "reserved_space", None) and retval.reserved_space < 0:
            raise KickstartParseError("Volume group reserved space must be a positive integer.", lineno=self.lineno)

        if getattr(retval, "reserved_percent", None) is not None and not 0 < retval.reserved_percent < 100:
            raise KickstartParseError("Volume group reserved space percentage must be between 1 and 99.", lineno=self.lineno)

        # the volgroup command can't be used together with the autopart command
        # due to the hard to debug behavior their combination introduces
        if self.handler.autopart.seen:
            errorMsg = _("The volgroup and autopart commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        # the same applies to the 'mount' command
        if hasattr(self.handler, "mount") and self.handler.mount.seen:
            errorMsg = _("The volgroup and mount commands can't be used at the same time")
            raise KickstartParseError(errorMsg, lineno=self.lineno)
        return retval

class F21_VolGroup(F16_VolGroup):
    def _getParser(self):
        op = F16_VolGroup._getParser(self)
        op.add_argument("--pesize", type=int, default=0, version=F21, help="""
                        Set the size of the physical extents in KiB.""")
        return op

class RHEL7_VolGroup(F21_VolGroup):
    pass

class RHEL8_VolGroup(F21_VolGroup):
    pass



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2015 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
import unittest
from pykickstart.version import F23
from pykickstart.commands.reqpart import F23_ReqPart
from tests.baseclass import CommandTest, CommandSequenceTest

class ReqPart_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = F23_ReqPart()
        self.assertEqual(cmd.reqpart, False)
        self.assertEqual(cmd.addBoot, False)

class F23_TestCase(CommandTest):
    command = "reqpart"

    def runTest(self):
        # pass
        self.assert_parse("reqpart", "reqpart\n")

        # pass
        self.assert_parse("reqpart --add-boot", "reqpart --add-boot\n")

class F23_AutopartReqpart_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F23

    def runTest(self):
        # fail - can't use both autopart and reqpart
        self.assert_parse_error("""
autopart
reqpart""")

class RHEL7_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

if __name__ == "__main__":
    unittest.main()



#
# Chris Lumens <clumens@redhat.com>
#
# Copyright 2009, 2013 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.commands import (
    authconfig,
    authselect,
    autopart,
    autostep,
    bootloader,
    btrfs,
    clearpart,
    cdrom,
    device,
    deviceprobe,
    displaymode,
    dmraid,
    driverdisk,
    module,
    eula,
    fcoe,
    firewall,
    firstboot,
    group,
    harddrive,
    hmc,
    ignoredisk,
    install,
    interactive,
    iscsi,
    iscsiname,
    key,
    keyboard,
    lang,
    langsupport,
    lilocheck,
    liveimg,
    logging,
    logvol,
    mediacheck,
    method,
    monitor,
    mount,
    mouse,
    multipath,
    network,
    nfs,
    nvdimm,
    ostreesetup,
    partition,
    raid,
    reqpart,
    realm,
    reboot,
    repo,
    rescue,
    rhsm,
    rootpw,
    selinux,
    services,
    skipx,
    snapshot,
    sshpw,
    sshkey,
    syspurpose,
    timezone,
    updates,
    upgrade,
    url,
    user,
    unsupported_hardware,
    vnc,
    volgroup,
    xconfig,
    zerombr,
    zfcp,
    zipl,
)



# Andy Lindeberg <alindebe@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from tests.baseclass import CommandTest
from pykickstart.errors import KickstartParseError


class FC3_TestCase(CommandTest):
    command = "displaymode"

    def runTest(self):
        # pass
        self.assert_parse("graphical", "graphical\n")
        self.assert_parse("text", "text\n")
        self.assert_parse("cmdline", "cmdline\n")

        # fail
        self.assert_parse_error("graphical --glitter=YES")
        self.assert_parse_error("graphical --shiny")
        self.assert_parse_error("graphical text")
        self.assert_parse_error("graphical cmdline")
        self.assert_parse_error("text --glitter=YES")
        self.assert_parse_error("text --shiny")
        self.assert_parse_error("text graphical")
        self.assert_parse_error("text cmdline")
        self.assert_parse_error("cmdline --glitter=YES")
        self.assert_parse_error("cmdline --shiny")
        self.assert_parse_error("cmdline graphical")
        self.assert_parse_error("cmdline text")

        # extra test coverage
        cmd = self.handler().commands["text"]
        self.assertIsNotNone(cmd._getParser())

        for mode in [None, 999, -1]:
            cmd.displayMode = mode
            self.assertEqual(cmd.__str__(), "")

        with self.assertRaises(KickstartParseError):
            cmd.currentCmd = None
            cmd.parse([])


class F26_TestCase(FC3_TestCase):
    def runTest(self):
        #pass
        self.assert_parse("text --non-interactive", "text --non-interactive\n")
        self.assert_parse("graphical --non-interactive", "graphical --non-interactive\n")

        #fail
        self.assert_parse_error("cmdline --non-interactive", KickstartParseError)
        self.assert_parse_error("text --non-interactive-test", KickstartParseError)
        self.assert_parse_error("text --non-interactive --test", KickstartParseError)
        self.assert_parse_error("graphical --non-interactive-test", KickstartParseError)
        self.assert_parse_error("graphical --non-interactive --test", KickstartParseError)

if __name__ == "__main__":
    unittest.main()



#
# Vendula Poncova <vponcova@redhat.com>
#
# Copyright 2017 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#
from pykickstart.base import KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import RHEL7
from pykickstart.i18n import _


class RHEL7_Hmc(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __str__(self):
        retval = KickstartCommand.__str__(self)
        if not self.seen:
            return retval

        retval += "# Use installation media via SE/HMC\nhmc\n"
        return retval

    def _getParser(self):
        return KSOptionParser(prog="hmc", description="""
                              Install from an installation medium via SE/HMC on
                              z Systems.""", version=RHEL7)

    def parse(self, args):
        if args:
            msg = _("Kickstart command %s does not take any arguments") % self.currentCmd
            raise KickstartParseError(msg, lineno=self.lineno)

        return self

class F28_Hmc(RHEL7_Hmc):
    pass



import unittest
from tests.baseclass import ParserTest

from pykickstart.errors import formatErrorMsg, KickstartError, KickstartParseError, \
    KickstartVersionError, _format_error_message


class ErrorMessage_TestCase(ParserTest):
    def runTest(self):
        # For now, just verify that calling formatErrorMsg with no message
        # returns something.  Digging in and checking what the message is
        # when we could be running "make check" in another language is hard.

        # NOTE: formatErrorMsg is replaced with _format_error_message.
        self.assertNotEqual(_format_error_message(47), "")

        # Function formatErrorMsg is deprecated and returns its arguments now.
        with self.assertWarns(DeprecationWarning):
            self.assertEqual(formatErrorMsg(47), (47, ""))

        with self.assertWarns(DeprecationWarning):
            self.assertEqual(formatErrorMsg(47, "OH NO!"), (47, "OH NO!"))


class Exception_TestCase(ParserTest):
    def runTest(self):
        self.assertEqual(str(KickstartError("OH NO!")), "OH NO!")
        self.assertEqual(str(KickstartParseError("OH NO!")), "OH NO!")
        self.assertEqual(str(KickstartVersionError("OH NO!")), "OH NO!")

        err = KickstartError()
        self.assertEqual(err.message, "")
        self.assertEqual(err.lineno, None)
        self.assertEqual(err.value, "")
        self.assertEqual(str(err), "")

        err = KickstartError("OH NO!")
        self.assertEqual(err.message, "OH NO!")
        self.assertEqual(err.lineno, None)
        self.assertEqual(err.value, "OH NO!")
        self.assertEqual(str(err), "OH NO!")

        err = KickstartError(lineno=0)
        self.assertEqual(err.message, "")
        self.assertEqual(err.lineno, 0)
        self.assertEqual(err.value, _format_error_message(msg="", lineno=0))
        self.assertEqual(str(err), _format_error_message(msg="", lineno=0))

        err = KickstartError(lineno=1)
        self.assertEqual(err.message, "")
        self.assertEqual(err.lineno, 1)
        self.assertEqual(err.value, _format_error_message(msg="", lineno=1))
        self.assertEqual(str(err), _format_error_message(msg="", lineno=1))

        err = KickstartError("OH NO!", lineno=0)
        self.assertEqual(err.message, "OH NO!")
        self.assertEqual(err.lineno, 0)
        self.assertEqual(err.value, _format_error_message(msg="OH NO!", lineno=0))
        self.assertEqual(str(err), _format_error_message(msg="OH NO!", lineno=0))

        err = KickstartError("OH NO!", lineno=1)
        self.assertEqual(err.message, "OH NO!")
        self.assertEqual(err.lineno, 1)
        self.assertEqual(err.value, _format_error_message(msg="OH NO!", lineno=1))
        self.assertEqual(str(err), _format_error_message(msg="OH NO!", lineno=1))

        err = KickstartError("OH NO!", lineno=1, formatting=True)
        self.assertEqual(err.message, "OH NO!")
        self.assertEqual(err.lineno, 1)
        self.assertEqual(err.value, _format_error_message(msg="OH NO!", lineno=1))
        self.assertEqual(str(err), _format_error_message(msg="OH NO!", lineno=1))

        err = KickstartError("OH NO!", lineno=1, formatting=False)
        self.assertEqual(err.message, "OH NO!")
        self.assertEqual(err.lineno, 1)
        self.assertEqual(err.value, "OH NO!")
        self.assertEqual(str(err), "OH NO!")

        with self.assertWarns(DeprecationWarning):
            err = KickstartError(formatErrorMsg(lineno=0))
            self.assertEqual(err.message, "")
            self.assertEqual(err.lineno, 0)
            self.assertEqual(err.value, _format_error_message(msg="", lineno=0))
            self.assertEqual(str(err), _format_error_message(msg="", lineno=0))

        with self.assertWarns(DeprecationWarning):
            err = KickstartError(formatErrorMsg(lineno=1))
            self.assertEqual(err.message, "")
            self.assertEqual(err.lineno, 1)
            self.assertEqual(err.value, _format_error_message(msg="", lineno=1))
            self.assertEqual(str(err), _format_error_message(msg="", lineno=1))

        with self.assertWarns(DeprecationWarning):
            err = KickstartError(formatErrorMsg(msg="OH NO!", lineno=0))
            self.assertEqual(err.message, "OH NO!")
            self.assertEqual(err.lineno, 0)
            self.assertEqual(err.value, _format_error_message(msg="OH NO!", lineno=0))
            self.assertEqual(str(err), _format_error_message(msg="OH NO!", lineno=0))

        with self.assertWarns(DeprecationWarning):
            err = KickstartError(formatErrorMsg(msg="OH NO!", lineno=1))
            self.assertEqual(err.message, "OH NO!")
            self.assertEqual(err.lineno, 1)
            self.assertEqual(err.value, _format_error_message(msg="OH NO!", lineno=1))
            self.assertEqual(str(err), _format_error_message(msg="OH NO!", lineno=1))


if __name__ == "__main__":
    unittest.main()



import os
import six
import unittest
import tempfile
from tests.baseclass import ParserTest

from pykickstart import constants
from pykickstart.errors import KickstartError, KickstartParseError

class Base_Include(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.includeKS = ""

    def setUp(self):
        ParserTest.setUp(self)

        (handle, self._path) = tempfile.mkstemp(prefix="include-", text=True)
        ks = self.includeKS
        if six.PY3:
            ks = ks.encode('utf-8')

        os.write(handle, ks)
        os.close(handle)

    def tearDown(self):
        ParserTest.tearDown(self)
        os.unlink(self._path)

class Include_Missing_TestCase(ParserTest):
    def runTest(self):
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, "%include")
        self.assertRaises(KickstartError, self.parser.readKickstartFromString, "%include /tmp/FILE_NOT_FOUND")

class Include_Missing_In_Section_TestCase(ParserTest):
    def runTest(self):
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, """
%packages
%include
%end""")

        self.assertRaises(KickstartError, self.parser.readKickstartFromString, """
%packages
%include /tmp/FILE_NOT_FOUND
%end""")

class Include_Packages_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
%%packages
%%include %s
-packageB
%%end
"""

        self.includeKS = """
packageA
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._path)

        self.assertEqual(len(self.handler.packages.packageList), 1)
        self.assertEqual(len(self.handler.packages.excludedList), 1)

class Include_Commands_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
rootpw 123456
%%include %s
text
"""

        self.includeKS = """
autopart
clearpart --all
zerombr
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._path)

        self.assertEqual(self.handler.rootpw.password, "123456")
        self.assertEqual(self.handler.displaymode.displayMode, constants.DISPLAY_MODE_TEXT)
        self.assertTrue(self.handler.autopart.autopart)
        self.assertEqual(self.handler.clearpart.type, constants.CLEARPART_TYPE_ALL)
        self.assertTrue(self.handler.zerombr.zerombr)

class Include_Whole_Script_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
%%include %s
"""

        self.includeKS = """
%pre
ls /tmp
%end
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._path)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify the script defaults.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/bin/sh")
        self.assertFalse(script.inChroot)
        self.assertEqual(script.lineno, 2)
        self.assertFalse(script.errorOnFail)
        self.assertEqual(script.type, constants.KS_SCRIPT_PRE)

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Include_Post_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
%%post
%%include %s
%%end
"""

        self.includeKS = """
ls /tmp
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._path)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify the script defaults.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/bin/sh")
        self.assertTrue(script.inChroot)
        self.assertEqual(script.lineno, 2)
        self.assertFalse(script.errorOnFail)
        self.assertEqual(script.type, constants.KS_SCRIPT_POST)

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Include_Pre_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
%%pre
%%include %s
%%end
"""

        self.includeKS = """
ls /tmp
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._path)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify the script defaults.
        script = self.handler.scripts[0]
        self.assertEqual(script.interp, "/bin/sh")
        self.assertFalse(script.inChroot)
        self.assertEqual(script.lineno, 2)
        self.assertFalse(script.errorOnFail)
        self.assertEqual(script.type, constants.KS_SCRIPT_PRE)

        # Also verify the body, which is the most important part.
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Include_URL_TestCase(Base_Include):
    def __init__(self, *args, **kwargs):
        Base_Include.__init__(self, *args, **kwargs)
        self.ks = """
%%include %s
"""

        self.includeKS = """
%pre
ls /tmp
%end
"""

    def setUp(self):
        super(Include_URL_TestCase, self).setUp()

        server = six.moves.BaseHTTPServer.HTTPServer(('127.0.0.1', 0), six.moves.SimpleHTTPServer.SimpleHTTPRequestHandler)
        httpd_port = server.server_port
        self._httpd_pid = os.fork()
        if self._httpd_pid == 0:
            os.chdir(os.path.dirname(self._path))
            server.serve_forever()

        self._url = 'http://127.0.0.1:%d/%s' % (httpd_port, os.path.basename(self._path))

    def tearDown(self):
        super(Include_URL_TestCase, self).tearDown()

        import signal
        os.kill(self._httpd_pid, signal.SIGTERM)

    def runTest(self):
        self.parser.readKickstartFromString(self.ks % self._url)
        self.assertEqual(len(self.handler.scripts), 1)

        # Verify that the script body came through
        script = self.handler.scripts[0]
        self.assertEqual(script.script.rstrip(), "ls /tmp")

class Include_Bad_URL_TestCase(Include_URL_TestCase):
    def runTest(self):
        # Add some garbage to the end of the URL and ensure it breaks
        six.assertRaisesRegex(self, KickstartError, "Error accessing URL",
                              self.parser.readKickstartFromString, self.ks % (self._url + "-garbage"))

if __name__ == "__main__":
    unittest.main()



#
# Martin Gracik <mgracik@redhat.com>
#
# Copyright 2009 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest
from pykickstart.version import F20
from pykickstart.errors import KickstartParseError
from tests.baseclass import CommandTest, CommandSequenceTest
from pykickstart.commands.partition import FC3_PartData, FC3_Partition

class Partition_TestCase(unittest.TestCase):
    def runTest(self):
        cmd = FC3_Partition()
        self.assertEqual(cmd.__str__(), '')

        data1 = FC3_PartData()
        data2 = FC3_PartData()

        # test that new objects are always equal
        self.assertEqual(data1, data2)
        self.assertNotEqual(data1, None)

        # test for objects difference
        for atr in ['mountpoint']:
            setattr(data1, atr, '')
            setattr(data2, atr, 'test')
            # objects that differ in only one attribute
            # are not equal
            self.assertNotEqual(data1, data2)
            self.assertNotEqual(data2, data1)
            setattr(data1, atr, '')
            setattr(data2, atr, '')


class FC3_TestCase(CommandTest):
    command = "partition"

    def __init__(self, *kargs, **kwargs):
        CommandTest.__init__(self, *kargs, **kwargs)
        self.bytesPerInode = ""

    def runTest(self):
        if "--bytes-per-inode" in self.optionList:
            self.bytesPerInode = " --bytes-per-inode=4096"

        # pass
        self.assert_parse("part /home", "part /home%s\n" % self.bytesPerInode)
        self.assert_parse("part /home --onpart=/dev/sda1", "part /home --onpart=sda1%s\n" % self.bytesPerInode)

        if "--start" in self.optionList:
            self.assert_parse("partition raid.1 --active --asprimary --start=0 --end=10 --fstype=ext3 --noformat",
                              "part raid.1 --active --asprimary --end=10 --fstype=\"ext3\" --noformat%s\n" % self.bytesPerInode)
            self.assert_parse("partition raid.1 --start=1 --end=10",
                              "part raid.1 --end=10 --start=1%s\n" % self.bytesPerInode)
        else:
            self.assert_parse("partition raid.1 --active --asprimary --fstype=ext3 --noformat",
                              "part raid.1 --active --asprimary --fstype=\"ext3\" --noformat%s\n" % self.bytesPerInode)

        self.assert_parse("part pv.1 --ondisk=sda --onpart=sda1 --recommended",
                          "part pv.1 --ondisk=sda --onpart=sda1 --recommended%s\n" % self.bytesPerInode)
        self.assert_parse("part pv.1 --ondrive=sda --usepart=sda1 --recommended",
                          "part pv.1 --ondisk=sda --onpart=sda1 --recommended%s\n" % self.bytesPerInode)
        self.assert_parse("part / --onbiosdisk=hda --size=100", "part / --onbiosdisk=hda --size=100%s\n" % self.bytesPerInode)
        self.assert_parse("part swap --grow --maxsize=100", "part swap --grow --maxsize=100%s\n" % self.bytesPerInode)

        # does not remove the /dev/ part
        self.assert_parse("part /usr --ondisk=/dev/sda --recommended --noformat --active",
                          "part /usr --active --noformat --ondisk=/dev/sda --recommended%s\n" % self.bytesPerInode)

        # fail
        # missing mountpoint
        self.assert_parse_error("part")
        self.assert_parse_error("part --ondisk=sda --size=100")

        # multiple mountpoint
        self.assert_parse_error("part /home / --ondisk=sda --size=100")

        int_params = ["size", "maxsize"]
        if "--start" in self.optionList:
            int_params += ["start", "end"]
        for opt in int_params:
            # integer argument required
            self.assert_parse_error("part / --%s=string" % opt)
            # value required
            self.assert_parse_error("part / --%s" % opt)

        for opt in ("fstype", "onbiosdisk", "ondisk", "ondrive", "onpart", "usepart"):
            # value required
            self.assert_parse_error("part / --%s" % opt)

        # only one argument allowed
        self.assert_parse_error("part / /home /usr")
        # unknown option
        self.assert_parse_error("part /home --unknown=value")
        self.assert_parse_error("part --bogus-option")

        parser = self.handler().commands["part"]
        pd = parser.parse(["/home"])
        self.assertFalse(pd == "")
        self.assertTrue(pd != "")

        # extra test coverage
        parser.partitions = [pd]
        if "--bytes-per-inode" in self.optionList:
            self.assertEqual(parser.__str__(), "# Disk partitioning information\npart /home --bytes-per-inode=4096\n")
        else:
            self.assertEqual(parser.__str__(), "# Disk partitioning information\npart /home\n")


class FC4_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        if "--bytes-per-inode" in self.optionList:
            # pass
            self.assert_parse("part /home --bytes-per-inode=2048 --fsoptions=blah --label=home",
                              "part /home --bytes-per-inode=2048 --fsoptions=\"blah\" --label=home\n")

            # fail
            # --bytes-per-inode requires int argument
            self.assert_parse_error("part /home --bytes-per-inode=string")
            self.assert_parse_error("part /home --bytes-per-inode")

        # missing required arguments
        for opt in ("--fsoptions", "--label"):
            if opt in self.optionList:
                self.assert_parse_error("part /home %s" % opt)

class RHEL5_TestCase(FC4_TestCase):
    def runTest(self):
        # run FC4 test case
        FC4_TestCase.runTest(self)

        # pass
        self.assert_parse("part / --encrypted --passphrase=blahblah",
                          "part / --bytes-per-inode=4096 --encrypted --passphrase=\"blahblah\"\n")

        # fail
        # missing required --passphrase argument
        self.assert_parse_error("part / --encrypted --passphrase")

        # extra test coverage
        data = self.handler().PartData()
        data.encrypted = True
        data.passphrase = ""
        self.assertEqual(data._getArgsAsStr(), " --bytes-per-inode=4096 --encrypted")

class F9_TestCase(FC3_TestCase):
    def runTest(self):
        # run FC3 test case
        FC3_TestCase.runTest(self)

        # pass
        self.assert_parse("part / --encrypted --passphrase=blahblah",
                          "part / --encrypted --passphrase=\"blahblah\"\n")
        self.assert_parse("part /home --fsprofile=blah", "part /home --fsprofile=\"blah\"\n")

        # deprecated
        self.assert_deprecated("part", "--bytes-per-inode")

        # fail
        # missing required --passphrase argument
        self.assert_parse_error("part / --encrypted --passphrase")
        # missing required --fsprofile argument
        self.assert_parse_error("part / --fsprofile")

class F12_TestCase(F9_TestCase):
    def runTest(self):
        # Run F9 test case
        F9_TestCase.runTest(self)

        # pass
        self.assert_parse("part / --escrowcert=\"http://x/y\"", "part /\n")
        self.assert_parse("part / --encrypted --backuppassphrase",
                          "part / --encrypted\n")
        self.assert_parse("part / --encrypted --escrowcert=\"http://x/y\"",
                          "part / --encrypted --escrowcert=\"http://x/y\"\n")
        self.assert_parse("part / --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase",
                          "part / --encrypted --escrowcert=\"http://x/y\" "
                          "--backuppassphrase\n")
        self.assert_parse("part / --encrypted --escrowcert=http://x/y",
                          "part / --encrypted --escrowcert=\"http://x/y\"\n")

        # fail
        self.assert_parse_error("part / --escrowcert")
        self.assert_parse_error("part / --escrowcert --backuppassphrase")
        self.assert_parse_error("part / --encrypted --escrowcert "
                                "--backuppassphrase")
        self.assert_parse_error("part / --backuppassphrase=False")
        self.assert_parse_error("part / --backuppassphrase=True")

class RHEL6_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)

        self.assert_parse("part / --encrypted --cipher=3-rot13",
                          "part / --encrypted --cipher=\"3-rot13\"\n")
        # Allowed here, but anaconda should complain.  Note how we throw out
        # cipher from the output if there's no --encrypted.
        self.assert_parse("part / --cipher=3-rot13",
                          "part /\n")

        self.assert_parse_error("part / --cipher")

        self.assert_parse("part swap --hibernation", "part swap --hibernation\n")
        self.assert_parse("part swap --recommended --hibernation")

        with self.assertRaises(KickstartParseError):
            parser = self.handler().commands["part"]
            parser.handler.autopart.seen = True
            parser.parse(["autopart"])

class F14_TestCase(F12_TestCase):
    def runTest(self):
        F12_TestCase.runTest(self)
        self.assert_removed("partition", "--bytes-per-inode")
        self.assert_removed("partition", "--start")
        self.assert_removed("partition", "--end")

class F17_TestCase(F14_TestCase):
    def runTest(self):
        F14_TestCase.runTest(self)
        self.assert_parse("part /foo --resize --size 500 --onpart=sda3",
                          "part /foo --onpart=sda3 --size=500 --resize\n")

        # no onpart or size
        self.assert_parse_error("part /foo --resize")

        # no onpart
        self.assert_parse_error("part /foo --size=999 --resize")

        # no size
        self.assert_parse_error("part /foo --onpart=LABEL=var --resize")

class F18_TestCase(F17_TestCase):
    def runTest(self):
        F17_TestCase.runTest(self)
        self.assert_parse("part swap --hibernation", "part swap --hibernation\n")
        self.assert_parse("part swap --recommended")
        self.assert_parse("part swap --recommended --hibernation")

        self.assert_parse("part / --encrypted --cipher=3-rot13",
                          "part / --encrypted --cipher=\"3-rot13\"\n")
        # Allowed here, but anaconda should complain.  Note how we throw out
        # cipher from the output if there's no --encrypted.
        self.assert_parse("part / --cipher=3-rot13",
                          "part /\n")

        self.assert_parse_error("part / --cipher")

class F20_TestCase(F18_TestCase):
    def runTest(self):
        F18_TestCase.runTest(self)

        self.assert_parse("part /tmp --fstype=tmpfs")

        # --grow and --maxsize isn't supported with tmpfs
        self.assert_parse_error("part /tmp --fstype=tmpfs --grow")
        self.assert_parse_error("part /tmp --fstype=tmpfs --maxsize=10")


class F20_Conflict_TestCase(CommandSequenceTest):
    def __init__(self, *args, **kwargs):
        CommandSequenceTest.__init__(self, *args, **kwargs)
        self.version = F20

    def runTest(self):
        self.assert_parse_error("""
autopart
part / --size=1024 --fstype=ext4""")

class F23_TestCase(F20_TestCase):
    def runTest(self):
        F20_TestCase.runTest(self)

        # pass
        self.assert_parse("part / --size=4096 --mkfsoptions=some,thing",
                          "part / --size=4096 --mkfsoptions=\"some,thing\"\n")

        # can't use --mkfsoptions if you're not formatting
        self.assert_parse_error("part / --size=4096 --mkfsoptions=some,thing --noformat")

        # can't use --mkfsoptions with --fsprofile
        self.assert_parse_error("part / --size=4096 --mkfsoptions=some,thing --fsprofile=PROFILE")

class RHEL7_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

class F29_TestCase(F23_TestCase):
    def runTest(self):
        F23_TestCase.runTest(self)

        self.assert_deprecated("part", "--active")
        self.assert_deprecated("partition", "--active")

        self.assert_parse("part / --encrypted --luks-version=luks2",
                          "part / --encrypted --luks-version=luks2\n")

        self.assert_parse("part / --encrypted --pbkdf=argon2i",
                          "part / --encrypted --pbkdf=argon2i\n")

        self.assert_parse("part / --encrypted --pbkdf-memory=256",
                          "part / --encrypted --pbkdf-memory=256\n")

        self.assert_parse("part / --encrypted --pbkdf-time=100",
                          "part / --encrypted --pbkdf-time=100\n")

        self.assert_parse("part / --encrypted --pbkdf-iterations=1000",
                          "part / --encrypted --pbkdf-iterations=1000\n")

        self.assert_parse_error("part / --encrypted --pbkdf-time=100 --pbkdf-iterations=1000")


class RHEL8_TestCase(F29_TestCase):
    def  runTest(self):
        F29_TestCase.runTest(self)
        self.assert_parse_error("part / --fstype=btrfs")

if __name__ == "__main__":
    unittest.main()



#
# Alexander Todorov <atodorov@redhat.com>
#
# Copyright 2016 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

import unittest

from pykickstart.base import DeprecatedCommand
from tests.baseclass import CommandTest

class F20_TestCase(CommandTest):
    command = "install"

    def runTest(self):
        # pass
        self.assert_parse("install", "install\n")
        self.assert_parse("install", "install\n")
        self.assert_parse("install --root-device=/dev/sda", "install\n")

        # upgrade is always false
        cmd = self.handler().commands[self.command]
        cmd.parse([])
        self.assertFalse(cmd.upgrade)

        # fail
        self.assert_parse_error("install --bad-flag")
        # --root-device requires argument
        self.assert_parse_error("install --root-device")
        self.assert_parse_error("install --root-device=\"\"")

class F29_TestCase(F20_TestCase):
    def runTest(self):
        # make sure we've been deprecated
        parser = self.getParser("install")
        self.assertTrue(issubclass(parser.__class__, DeprecatedCommand))

        # make sure we are still able to parse it
        self.assert_parse("install")

class RHEL8_TestCase(F29_TestCase):
    pass

if __name__ == "__main__":
    unittest.main()



import unittest
import warnings

from tests.baseclass import ParserTest

from pykickstart import sections
from pykickstart.errors import KickstartParseError, KickstartParseWarning


class RawSection(sections.Section):
    def __init__(self, handler, **kwargs):
        sections.Section.__init__(self, handler, **kwargs)
        self.handler.raw = ""
        self.sectionOpen = "%raw"

    def handleLine(self, line):
        if not self.handler:
            return

        self.handler.raw += line

class New_Section_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%raw
1234
abcd
%end
"""

    def runTest(self):
        self.parser.registerSection(RawSection(self.parser.handler))
        self.parser.readKickstartFromString(self.ks)

        # Verify the contents of the custom section were saved.
        self.assertEqual(self.parser.handler.raw, "1234\nabcd\n")

class Unknown_New_Section_1_TestCase(New_Section_TestCase):
    def runTest(self):
        # pykickstart doesn't understand the %raw section, and we've not
        # told it to ignore these errors.
        self.assertRaises(KickstartParseError, self.parser.readKickstartFromString, self.ks)

class Unknown_New_Section_2_TestCase(New_Section_TestCase):
    def runTest(self):
        # pykickstart doesn't understand the %raw section, but we've told
        # it to ignore that.  There's not an "assertDoesntRaise" function,
        # so we just call it and if anything goes wrong it'll FAIL.
        self.parser.unknownSectionIsFatal = False
        self.parser.readKickstartFromString(self.ks)

class Ignored_Section_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
%addon
whatever
%end

%anaconda
whatever2
%end
"""

    def runTest(self):
        # pykickstart recognizes these sections, but doesn't do anything with them.
        # Be as strict as possible by turning warnings into errors, to make sure we
        # don't even warn about them.
        warnings.simplefilter("error", category=KickstartParseWarning)
        self.parser.readKickstartFromString(self.ks)
        warnings.resetwarnings()

if __name__ == "__main__":
    unittest.main()



#
# Jiri Konecny <jkonecny@redhat.com>
#
# Copyright 2005, 2006, 2007, 2008, 2012, 2016, 2017 Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use, modify,
# copy, or redistribute it subject to the terms and conditions of the GNU
# General Public License v.2.  This program is distributed in the hope that it
# will be useful, but WITHOUT ANY WARRANTY expressed or implied, including the
# implied warranties of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51
# Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  Any Red Hat
# trademarks that are incorporated in the source code or documentation are not
# subject to the GNU General Public License and may only be used or replicated
# with the express permission of Red Hat, Inc.
#

from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.constants import SNAPSHOT_WHEN_POST_INSTALL, SNAPSHOT_WHEN_PRE_INSTALL
from pykickstart.version import F26

from pykickstart.i18n import _


class F26_SnapshotData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.name = kwargs.get("name", "")
        self.origin = kwargs.get("origin", "")
        self.when = kwargs.get("when", None)

    def __eq__(self, y):
        if not y:
            return False
        return (self.name == y.name and
                self.origin == y.origin and
                self.when == y.when)

    def __ne__(self, y):
        return not self == y

    def _getArgsAsStr(self):
        retval = ""

        if self.when == SNAPSHOT_WHEN_POST_INSTALL:
            retval += "--when=post-install"
        elif self.when == SNAPSHOT_WHEN_PRE_INSTALL:
            retval += "--when=pre-install"

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += ("snapshot %s --name=%s %s" % (self.origin, self.name, self._getArgsAsStr()))
        return retval.strip() + "\n"


class F26_Snapshot(KickstartCommand):
    removedKeywords = KickstartCommand.removedKeywords
    removedAttrs = KickstartCommand.removedAttrs

    def __init__(self, writePriority=140, *args, **kwargs):
        KickstartCommand.__init__(self, writePriority, *args, **kwargs)

        self.snapshotList = kwargs.get("snapshotList", [])
        self.whenMap = { "post-install": SNAPSHOT_WHEN_POST_INSTALL,
                         "pre-install": SNAPSHOT_WHEN_PRE_INSTALL }
        self.op = self._getParser()

    def __str__(self):
        retval = ""

        for snapshot in self.snapshotList:
            retval += snapshot.__str__()

        return retval

    def _when_cb(self, value):
        if value.lower() in self.whenMap:
            return self.whenMap[value.lower()]
        else:
            msg = _("Invalid snapshot when parameter: %s") % value
            raise KickstartParseError(msg, lineno=self.lineno)

    def _getParser(self):
        op = KSOptionParser(prog="snapshot", version=F26, description="""
                            Create an LVM snapshot for devices on an LVM thin pool.""")
        op.add_argument("--name", metavar="<snapshot_name>", version=F26, required=True,
                        help="""
                        Name of the newly created snapshot.""")
        # Show all possible options in meta message
        meta_msg = "<%s>" % ("|".join(self.whenMap.keys()))
        op.add_argument("--when", metavar=meta_msg, type=self._when_cb, version=F26,
                        required=True, help="""
                        You can specify two possible values: ``pre-install`` and ``post-install``.
                        When the ``pre-install`` value is used the snapshot is created before
                        the installation but after the ``%%pre`` section is run.
                        When the ``post-install`` value is used the snapshot is created after
                        the installation is done and after the ``%%post`` section is run.""")
        return op

    def parse(self, args):
        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if len(extra) == 0:
            msg = _("Snapshot origin must be specified!")
            raise KickstartParseError(msg, lineno=self.lineno)
        elif len(extra) > 1:
            msg = _("Snapshot origin can be specified only once!")
            raise KickstartParseError(msg, lineno=self.lineno)

        snap_data = self.dataClass()   # pylint: disable=not-callable
        self.set_to_obj(ns, snap_data)
        snap_data.lineno = self.lineno
        snap_data.origin = extra[0]

        # Check for duplicates
        if snap_data.name in [snap.name for snap in self.dataList()]:
            msg = (_("Snapshot with the name %s has been already defined!") % snap_data.name)
            raise KickstartParseError(msg, lineno=self.lineno)

        if snap_data.when is None:
            msg = _("Snapshot \"when\" parameter must be specified!")
            raise KickstartParseError(msg, lineno=self.lineno)

        groups = snap_data.origin.split('/')
        if len(groups) != 2 or len(groups[0]) == 0 or len(groups[1]) == 0:
            msg = (_("Snapshot origin %s must be specified by VG/LV!") % snap_data.origin)
            raise KickstartParseError(msg, lineno=self.lineno)

        # Check if value in a '--when' param is valid
        if snap_data.when != "" and snap_data.when not in self.whenMap.values():
            msg = (_("Snapshot when param must have one of these values %s!") % self.whenMap.keys())
            raise KickstartParseError(msg, lineno=self.lineno)
        return snap_data

    def dataList(self):
        return self.snapshotList

    @property
    def dataClass(self):
        return self.handler.SnapshotData



import unittest
import warnings

from pykickstart.parser import KickstartParser
from pykickstart.version import makeVersion
from tests.baseclass import ParserTest
from pykickstart.handlers import control
from pykickstart.base import DeprecatedCommand

class Platform_Comment_TestCase(ParserTest):
    def __init__(self, *args, **kwargs):
        ParserTest.__init__(self, *args, **kwargs)
        self.ks = """
#platform=x86_64
autopart
"""

    def runTest(self):
        self.parser.readKickstartFromString(self.ks)
        self.assertEqual(self.handler.platform, "x86_64")

class WritePriority_TestCase(unittest.TestCase):
    def runTest(self):
        for _version, _map in control.commandMap.items():
            for _name, command_class in _map.items():
                cmd = command_class()
                if issubclass(cmd.__class__, DeprecatedCommand):
                    self.assertEqual(None, cmd.writePriority, command_class)
                elif _name in ['bootloader', 'lilo', 'zipl']:
                    self.assertEqual(10, cmd.writePriority, command_class)
                elif _name in ['multipath']:
                    self.assertEqual(50, cmd.writePriority, command_class)
                elif _name in ['dmraid']:
                    self.assertEqual(60, cmd.writePriority, command_class)
                elif _name in ['iscsiname']:
                    self.assertEqual(70, cmd.writePriority, command_class)
                elif _name in ['fcoe', 'zfcp', 'iscsi']:
                    self.assertEqual(71, cmd.writePriority, command_class)
                elif _name in ['nvdimm']:
                    self.assertEqual(80, cmd.writePriority, command_class)
                elif _name in ['autopart', 'reqpart']:
                    self.assertEqual(100, cmd.writePriority, command_class)
                elif _name in ['zerombr']:
                    self.assertEqual(110, cmd.writePriority, command_class)
                elif _name in ['clearpart']:
                    self.assertEqual(120, cmd.writePriority, command_class)
                elif _name in ['partition', 'part']:
                    self.assertEqual(130, cmd.writePriority, command_class)
                elif _name in ['raid']:
                    self.assertEqual(131, cmd.writePriority, command_class)
                elif _name in ['volgroup', 'btrfs']:
                    self.assertEqual(132, cmd.writePriority, command_class)
                elif _name in ['logvol']:
                    self.assertEqual(133, cmd.writePriority, command_class)
                elif _name in ['snapshot']:
                    self.assertEqual(140, cmd.writePriority, command_class)
                else:
                    self.assertEqual(0, cmd.writePriority, command_class)

class DeprecatedCommandsParsing_TestCase(unittest.TestCase):
    def runTest(self):
        for version, command_map in control.commandMap.items():

            handler = makeVersion(version)
            parser = KickstartParser(handler)

            for command_name, command_class in command_map.items():
                if not issubclass(command_class, DeprecatedCommand):
                    continue

                with warnings.catch_warnings(record=True):
                    # The deprecated commands should be ignored with
                    # a warning when they are parsed. Make sure that
                    # they will not cause any errors.
                    parser.readKickstartFromString(command_name)

if __name__ == "__main__":
    unittest.main()



#
# Copyright (C) 2017  Red Hat, Inc.
#
# This copyrighted material is made available to anyone wishing to use,
# modify, copy, or redistribute it subject to the terms and conditions of
# the GNU General Public License v.2, or (at your option) any later version.
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY expressed or implied, including the implied warranties of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
# Public License for more details.  You should have received a copy of the
# GNU General Public License along with this program; if not, write to the
# Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.  Any Red Hat trademarks that are incorporated in the
# source code or documentation are not subject to the GNU General Public
# License and may only be used or replicated with the express permission of
# Red Hat, Inc.
#
# Red Hat Author(s): Vratislav Podzimek <vpodzime@redhat.com>
#

from pykickstart.base import BaseData, KickstartCommand
from pykickstart.errors import KickstartParseError
from pykickstart.options import KSOptionParser
from pykickstart.version import F27

from pykickstart.i18n import _

class F27_MountData(BaseData):
    removedKeywords = BaseData.removedKeywords
    removedAttrs = BaseData.removedAttrs

    def __init__(self, *args, **kwargs):
        BaseData.__init__(self, *args, **kwargs)
        self.mount_point = kwargs.get("mount_point", "")
        self.device = kwargs.get("device", "")
        self.format = kwargs.get("format", "")
        self.reformat = kwargs.get("reformat", False)
        self.mkfs_opts = kwargs.get("mkfs_opts", "")
        self.mount_opts = kwargs.get("mount_opts", "")

    def __eq__(self, other):
        if not other:
            return False

        return self.mount_point == other.mount_point

    def __ne__(self, other):
        return not self == other

    def _getArgsAsStr(self):
        retval = "%s" % self.device

        retval += " %s" % self.mount_point or "none"

        if self.reformat:
            retval += " --reformat"
            if self.format:
                retval += "=%s" % self.format

        if self.mkfs_opts:
            retval += " --mkfsoptions=\"%s\"" % self.mkfs_opts

        if self.mount_opts:
            retval += " --mountoptions=\"%s\"" % self.mount_opts

        return retval

    def __str__(self):
        retval = BaseData.__str__(self)
        retval += "mount %s\n" % self._getArgsAsStr()
        return retval


class F27_Mount(KickstartCommand):
    """The 'mount' kickstart command"""

    def __init__(self, *args, **kwargs):
        KickstartCommand.__init__(self, *args, **kwargs)
        self.op = self._getParser()
        self.mount_points = kwargs.get("mount_points") or list()

    def __str__(self):
        retval = KickstartCommand.__str__(self)

        for mpoint in self.mount_points:
            retval += str(mpoint)

        if retval != "":
            retval = "# Mount points configuration\n" + retval + "\n"

        return retval

    def _getParser(self):
        op = KSOptionParser(prog="mount", description="""
                            Assigns a mount point to a block device and
                            optionally reformats it to a given format. It at
                            least requires a device and a mount point where the
                            mount point can be ``none`` in case the format on
                            the device is not mountable or in case the device
                            should just be reformatted.

                            The difference between this command and the other
                            commands for storage configuration (``part``,
                            ``logvol``,...) is that it doesn't require the whole
                            storage stack to be described in the kickstart
                            file. The user just needs to make sure that the
                            specified block device exists in the system. The
                            installer doesn't necessarily have to know all the
                            details about of the given device. If, on the other
                            hand, the installer is supposed to **create** the
                            storage stack with all the devices mounted at
                            various places, the ``part``, ``logvol``, ``raid``,
                            etc. commands have to be used.
                            """, version=F27)

        op.add_argument("device", metavar="<device>", nargs=1, version=F27,
                        help="""The block device to mount""")
        op.add_argument("mntpoint", metavar="<mntpoint>", nargs=1, version=F27,
                        help="""
                        The ``<mntpoint>`` is where the <device> will be
                        mounted.  Must be a valid mount point, for example
                        ``/``, ``/usr``, ``/home``, or ``none`` if the device
                        cannot (e.g. swap) or should not be mounted.""")

        op.add_argument("--reformat", dest="reformat", nargs="?", version=F27, const=True, default=False,
                        help="Specifies the new format (e.g. a file system) for the device.")
        op.add_argument("--mkfsoptions", dest="mkfs_opts", action="store", version=F27,
                        help="""
                        Specifies additional parameters to be passed to the
                        program that makes a filesystem on this partition. No
                        processing is done on the list of arguments, so they
                        must be supplied in a format that can be passed directly
                        to the mkfs program.  This means multiple options should
                        be comma-separated or surrounded by double quotes,
                        depending on the filesystem.""")
        op.add_argument("--mountoptions", dest="mount_opts", action="store", version=F27,
                        help="""
                        Specifies a free form string of options to be used when
                        mounting the filesystem. This string will be copied into
                        the /etc/fstab file of the installed system and should
                        be enclosed in quotes.""")

        return op

    def parse(self, args):
        # the 'mount' command can't be used together with any other
        # partitioning-related command
        conflicting_command = None

        # seen indicates that the corresponding
        # command has been seen in kickstart
        if self.handler.autopart.seen:
            conflicting_command = "autopart"
        if self.handler.partition.seen:
            conflicting_command = "part/partition"
        elif self.handler.raid.seen:
            conflicting_command = "raid"
        elif self.handler.volgroup.seen:
            conflicting_command = "volgroup"
        elif self.handler.logvol.seen:
            conflicting_command = "logvol"
        elif hasattr(self.handler, "reqpart") and self.handler.reqpart.seen:
            conflicting_command = "reqpart"

        if conflicting_command:
            # allow for translation of the error message
            errorMsg = _("The '%s' and 'mount' commands can't be used at the same time") % \
                         conflicting_command
            raise KickstartParseError(errorMsg, lineno=self.lineno)

        (ns, extra) = self.op.parse_known_args(args=args, lineno=self.lineno)

        if extra:
            mapping = {"command": "mount", "options": extra}
            raise KickstartParseError(_("Unexpected arguments to %(command)s command: %(options)s") % mapping, lineno=self.lineno)

        md = self.dataClass()  # pylint: disable=not-callable
        self.set_to_obj(ns, md)
        md.lineno = self.lineno
        md.device = ns.device[0]
        md.mount_point = ns.mntpoint[0]

        if md.mount_point.lower() != "none" and not md.mount_point.startswith("/"):
            raise KickstartParseError(_("Invalid mount point '%s' given") % md.mount_point, lineno=self.lineno)

        if md.reformat is False and md.mkfs_opts:
            raise KickstartParseError(_("'--mkfsoptions' requires --reformat"), lineno=self.lineno)

        # The semantics is as follows:
        #   --reformat          -> just reformat with the same format as existing
        #   --reformat=SOME_FMT -> reformat to given format
        #   no '--reformat'     -> don't reformat
        #
        # md.reformat can either be 'False' (not specified), 'True' (just
        # '--reformat') or a non-empty string ('--reformat=FORMAT'). Only the
        # last case requires special treatment.
        if md.reformat and md.reformat is not True:
            # a new format given
            md.format = md.reformat
            md.reformat = True

        return md

    def dataList(self):
        return self.mount_points

    @property
    def dataClass(self):
        return self.handler.MountData
